###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Test of the ProductionRequest and Production modules."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# pylint: disable=line-too-long,invalid-name,protected-access,missing-docstring

import unittest
from mock import Mock, MagicMock

from LHCbDIRAC.BookkeepingSystem.Client.test.mock_BookkeepingClient import BookkeepingClientFake, \
    stepMC, stepMC2, stepStripp, mergeStep, mergeStepBHADRON, step1Dict, step2Dict, step124620Dict, step124621Dict, \
    step125080, stepHistoMergingDict, mergeStepCALIBRA, mergeStepPIDMDST
from LHCbDIRAC.ProductionManagementSystem.Client.ProductionRequest import ProductionRequest, _splitIntoProductionSteps
from LHCbDIRAC.ProductionManagementSystem.Client.Production import Production

prodsDict = {1: {'productionType': 'DataStripping',
                 'stepsInProd': [123, 456],
                 'bkQuery': 'Full',
                 'removeInputsFlag': False,
                 'tracking': 0,
                 'outputSE': {'T1': 'SE1'},
                 'priority': 5,
                 'cpu': 1000000,
                 'outputFileMask': '',
                 'outputFileStep': '',
                 'input': [],
                 'target': '',
                 'groupSize': 1,
                 'plugin': 'ByRun',
                 'inputDataPolicy': 'download',
                 'derivedProduction': 0,
                 'transformationFamily': 0,
                 'previousProd': None,
                 'stepsInProd-ProdName': ["123['SDST']", "456['BHADRON.DST']"],
                 'events': -1,
                 'multicore': 'False',
                 'processors': (4, 4),
                 'outputMode': 'Any',
                 'ancestorDepth': 0
                 },

             2: {'productionType': 'Merge',
                 'stepsInProd': [456],
                 'bkQuery': 'fromPreviousProd',
                 'removeInputsFlag': True,
                 'tracking': 1,
                 'outputSE': {'T1': 'SE1'},
                 'priority': 8,
                 'cpu': 300000,
                 'outputFileMask': '',
                 'outputFileStep': '',
                 'input': [],
                 'target': '',
                 'groupSize': 1,
                 'plugin': 'BySize',
                 'inputDataPolicy': 'download',
                 'derivedProduction': 0,
                 'transformationFamily': 0,
                 'previousProd': 1,
                 'stepsInProd-ProdName': ["456['CALIBRATION.DST']"],
                 'events': -1,
                 'multicore': 'False',
                 'processors': (2, 2),
                 'outputMode': 'Local',
                 'ancestorDepth': 0
                 },

             3: {'productionType': 'Merge',
                 'stepsInProd': [456],
                 'bkQuery': 'fromPreviousProd',
                 'removeInputsFlag': True,
                 'tracking': 1,
                 'outputSE': {'T1': 'SE1'},
                 'priority': 9,
                 'cpu': 10000,
                 'outputFileMask': '',
                 'outputFileStep': '',
                 'input': [],
                 'target': '',
                 'groupSize': 2,
                 'plugin': 'BySize',
                 'inputDataPolicy': 'download',
                 'derivedProduction': 0,
                 'transformationFamily': 0,
                 'previousProd': 1,
                 'stepsInProd-ProdName': ["456['PID.MDST']"],
                 'events': -1,
                 'multicore': 'True',
                 'processors': (0, 2),
                 'outputMode': 'Any',
                 'ancestorDepth': 0
                 },

             4: {'productionType': 'Merge',
                 'stepsInProd': [789],
                 'bkQuery': 'fromPreviousProd',
                 'removeInputsFlag': True,
                 'tracking': 1,
                 'outputSE': {'T1': 'SE1'},
                 'priority': 9,
                 'cpu': 1000,
                 'outputFileMask': '',
                 'outputFileStep': '',
                 'input': [],
                 'target': '',
                 'groupSize': 3,
                 'plugin': 'BySize',
                 'inputDataPolicy': 'download',
                 'derivedProduction': 0,
                 'transformationFamily': 0,
                 'previousProd': 1,
                 'stepsInProd-ProdName': ["789['DAVINCIHIST', 'BRUNELHIST']"],
                 'events': -1,
                 'multicore': 'True',
                 'processors': (0, 0),
                 'outputMode': 'Any',
                 'ancestorDepth': 0
                 }
             }


class ClientTestCase(unittest.TestCase):
  """Base class for the Client test cases."""

  def setUp(self):

    self.bkClientMock = Mock()
    self.diracProdIn = Mock()
    self.diracProdIn.launchProduction.return_value = {'OK': True, 'Value': 321}

    self.bkClientFake = BookkeepingClientFake()

    self.maxDiff = None


class ProductionRequestSuccess(ClientTestCase):

  def test__mcSpecialCase(self):

    # prepare the test case
    prod = Production()
    prod.setParameter('ProcessingType', 'JDL', 'Test', 'ProductionGroupOrType')
    prod.addApplicationStep(stepDict=stepMC,
                            modulesList=['GaudiApplication'])
    prod.addApplicationStep(stepDict=stepMC2,
                            inputData='previousStep',
                            modulesList=['GaudiApplication'])
    prod.priority = '1'
    prod.addFinalizationStep()
    prod.setFileMask('', '4')
    for par in prod.LHCbJob.workflow.parameters:
      if par.getName() == 'Site':
        self.assertEqual(par.value, 'ANY')
      if par.getName() == 'outputDataFileMask':
        self.assertEqual(par.value, '')

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.tc = MagicMock()
    pr.opsH = MagicMock()
    pr.opsH.getValue.return_value = 'testValue'
    prod.LHCbJob._siteSet = {'testValue'}

    pr._modifyAndLaunchMCXML(prod, {'tracking': 0})
    for par in prod.LHCbJob.workflow.parameters:
      if par.getName() == 'Site':
        self.assertEqual(par.value, 'testValue')
      if par.getName() == 'numberOfEvents':
        self.assertEqual(par.value, 'testValue')
      if par.getName() == 'listoutput':
        self.assertTrue('gausshist' in dict(par.value).values())
      if par.getName() == 'outputDataStep':
        self.assertEqual(par.value, '1;4')
      if par.getName() == 'outputDataFileMask':
        self.assertEqual(par.value, 'GAUSSHIST;DST')

    # # re-prepare the test case
    prod = Production()
    prod.setParameter('ProcessingType', 'JDL', 'Test', 'ProductionGroupOrType')
    prod.addApplicationStep(stepDict=step1Dict,
                            inputData='previousStep',
                            modulesList=['GaudiApplication'])
    prod.addFinalizationStep()
    prod.setFileMask('', ['4'])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.tc = MagicMock()
    pr.opsH = MagicMock()
    pr.opsH.getValue.return_value = 'testValue'
    prod.LHCbJob._siteSet = {'testValue'}

    # # re-run
    pr._mcSpecialCase(prod, {'tracking': 0})
    for par in prod.LHCbJob.workflow.parameters:
      if par.getName() == 'Site':
        self.assertEqual(par.value, 'ANY')
      if par.getName() == 'listoutput':
        self.assertTrue('gausshist' not in dict(par.value).values())
      if par.getName() == 'outputDataStep':
        self.assertEqual(par.value, '4')
      if par.getName() == 'outputDataFileMask':
        self.assertEqual(par.value, '')
      if par.getName() == 'Priority':
        self.assertEqual(par.value, '1')

    # And again re-prepare the test case
    prod = Production()
    prod.setParameter('ProcessingType', 'JDL', 'Test', 'ProductionGroupOrType')
    prod.addApplicationStep(stepDict=step1Dict,
                            inputData='previousStep',
                            modulesList=['GaudiApplication'])
    prod.addFinalizationStep()
    prod.setFileMask('')

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.tc = MagicMock()
    pr.opsH = MagicMock()
    pr.opsH.getValue.return_value = 'testValue'
    prod.LHCbJob._siteSet = {'testValue'}

    pr._mcSpecialCase(prod, {'tracking': 0})
    for par in prod.LHCbJob.workflow.parameters:
      if par.getName() == 'Site':
        self.assertEqual(par.value, 'ANY')
      if par.getName() == 'listoutput':
        self.assertTrue('gausshist' not in dict(par.value).values())
      if par.getName() == 'outputDataFileMask':
        self.assertEqual(par.value, '')
      if par.getName() == 'Priority':
        self.assertEqual(par.value, '1')

  def test_resolveStepsSuccess(self):

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.stepsList = ['123']
    pr.outputVisFlag = [{'1': 'N'}]
    pr.compressionLvl = ['LOW']
    pr.resolveSteps()
    expected = dict(step1Dict)
    expected['OptionFiles'] = 'optsFiles;$APPCONFIGOPTS/Persistency/Compression-ZLIB-1.py'
    self.assertEqual(pr.stepsListDict, [expected])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.stepsList = ['125080']  # this one already has a compression level
    pr.outputVisFlag = [{'1': 'N'}]
    pr.resolveSteps()
    expected = dict(step125080)
    expectedOptionFiles = '$APPCONFIGOPTS/Gauss/Sim08-Beam4000GeV-mu100-2012-nu2.5.py;'
    expectedOptionFiles += '$DECFILESROOT/options/11102400.py;$LBPYTHIA8ROOT/options/Pythia8.py;'
    expectedOptionFiles += '$APPCONFIGOPTS/Gauss/G4PL_FTFP_BERT_EmNoCuts.py;'
    expected['OptionFiles'] = expectedOptionFiles
    self.assertEqual(pr.stepsListDict, [expected])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.stepsList = ['125080']  # this one already has a compression level
    pr.compressionLvl = ['LOW']
    pr.outputVisFlag = [{'1': 'N'}]
    pr.resolveSteps()
    expected = dict(step125080)
    self.assertEqual(pr.stepsListDict, [expected])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.stepsList = ['125080']  # this one already has a compression level
    pr.compressionLvl = ['HIGH']  # this will replace it
    pr.outputVisFlag = [{'1': 'N'}]
    pr.resolveSteps()
    expected = dict(step125080)
    expectedOptionFiles = '$APPCONFIGOPTS/Gauss/Sim08-Beam4000GeV-mu100-2012-nu2.5.py;'
    expectedOptionFiles += '$DECFILESROOT/options/11102400.py;'
    expectedOptionFiles += '$LBPYTHIA8ROOT/options/Pythia8.py;'
    expectedOptionFiles += '$APPCONFIGOPTS/Gauss/G4PL_FTFP_BERT_EmNoCuts.py;'
    expected['OptionFiles'] = expectedOptionFiles
    self.assertEqual(pr.stepsListDict, [expected])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.stepsList = ['123', '456']
    pr.compressionLvl = ['LOW', 'HIGH']
    pr.outputVisFlag = [{'1': 'N'}, {'2': 'N'}]
    expected1 = dict(step1Dict)
    expected1['OptionFiles'] = 'optsFiles;$APPCONFIGOPTS/Persistency/Compression-ZLIB-1.py'
    expected2 = dict(step2Dict)
    expected2['OptionFiles'] = 'optsFiles;$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py'
    pr.resolveSteps()
    self.assertEqual(pr.stepsListDict, [expected1, expected2])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.stepsList = ['123', '456', '789']
    pr.compressionLvl = ['LOW', 'HIGH', 'HIGH']
    pr.outputVisFlag = [{'1': 'N'}, {'2': 'N'}, {'3': 'Y'}]
    expected1 = dict(step1Dict)
    expected1['OptionFiles'] = 'optsFiles;$APPCONFIGOPTS/Persistency/Compression-ZLIB-1.py'
    expected2 = dict(step2Dict)
    expected2['OptionFiles'] = 'optsFiles;$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py'
    expected3 = dict(stepHistoMergingDict)
    expected3['OptionFiles'] = 'optsFiles;$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py'
    pr.resolveSteps()
    self.assertEqual(pr.stepsListDict, [expected1, expected2, expected3])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.stepsList = ['124620', '124621']
    pr.compressionLvl = ['LOW', 'HIGH']
    pr.outputVisFlag = [{'1': 'N'}, {'2': 'Y'}]
    expected1 = dict(step124620Dict)
    expected1['OptionFiles'] = '$APPCONFIGOPTS/Boole/Default.py;$APPCONFIGOPTS/Boole/DataType-2012.py;'
    expected2 = dict(step124621Dict)
    expected2['OptionFiles'] = 'someBrunelOptions'
    pr.resolveSteps()
    self.assertEqual(pr.stepsListDict, [expected1, expected2])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.stepsList = ['123', '456', '', '']
    pr.outputVisFlag = [{'1': 'N'}, {'2': 'N'}]
    pr.compressionLvl = ['LOW', 'LOW', 'HIGH', 'HIGH']
    pr.resolveSteps()
    self.assertEqual(pr.stepsListDict, [{'StepId': 123, 'StepName': 'Stripping14-Stripping',
                                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v2r2',
                                         'ExtraOptions': '',
                                         'OptionFiles': 'optsFiles;$APPCONFIGOPTS/Persistency/Compression-ZLIB-1.py',
                                         'Visible': 'Yes', 'ExtraPackages': 'eps',
                                         'ProcessingPass': 'procPass', 'OptionsFormat': '', 'SystemConfig': '',
                                         'DDDB': '', 'CONDDB': '123456', 'DQTag': '', 'isMulticore': 'N',
                                         'prodStepID': "123['SDST']", 'mcTCK': '',
                                         'visibilityFlag': [{'Visible': 'N', 'FileType': 'BHADRON.DST'},
                                                            {'Visible': 'N', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesIn': ['SDST'],
                                         'fileTypesOut':['BHADRON.DST', 'CALIBRATION.DST']},
                                        {'StepId': 456, 'StepName': 'Merge',
                                         'ApplicationName': 'LHCb', 'ApplicationVersion': 'v1r2', 'ExtraOptions': '',
                                         'OptionFiles': 'optsFiles;$APPCONFIGOPTS/Persistency/Compression-ZLIB-1.py',
                                         'Visible': 'Yes', 'ExtraPackages': 'eps',
                                         'ProcessingPass': 'procPass', 'OptionsFormat': '',
                                         'SystemConfig': 'x86_64-slc6-gcc49-opt',
                                         'prodStepID': "456['BHADRON.DST', 'CALIBRATION.DST']",
                                         'DDDB': '', 'CONDDB': '123456', 'DQTag': '', 'isMulticore': 'N', 'mcTCK': '',
                                         'visibilityFlag': [{'Visible': 'N', 'FileType': 'BHADRON.DST'},
                                                            {'Visible': 'N', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesIn': ['BHADRON.DST', 'CALIBRATION.DST'],
                                         'fileTypesOut':['BHADRON.DST', 'CALIBRATION.DST']}
                                        ])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.stepsList = ['123']
    pr.outputVisFlag = [{'1': 'N'}]
    pr.resolveSteps()
    self.assertEqual(pr.stepsListDict,
                     [{'StepId': 123,
                       'StepName': 'Stripping14-Stripping',
                       'ApplicationName': 'DaVinci',
                       'ApplicationVersion': 'v2r2',
                       'ExtraOptions': '',
                       'OptionFiles': 'optsFiles',
                       'Visible': 'Yes',
                       'ExtraPackages': 'eps',
                       'ProcessingPass': 'procPass',
                       'OptionsFormat': '',
                       'SystemConfig': '',
                       'DDDB': '',
                       'CONDDB': '123456',
                       'DQTag': '',
                       'isMulticore': 'N',
                       'prodStepID': "123['SDST']",
                       'mcTCK': '',
                       'visibilityFlag': [{'Visible': 'N',
                                           'FileType': 'BHADRON.DST'},
                                          {'Visible': 'N',
                                           'FileType': 'CALIBRATION.DST'}],
                         'fileTypesIn': ['SDST'],
                         'fileTypesOut':['BHADRON.DST',
                                         'CALIBRATION.DST']}])
    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.compressionLvl = ['HIGH']
    pr.stepsList = ['123']
    pr.outputVisFlag = [{'1': 'N'}]
    pr.resolveSteps()
    self.assertEqual(pr.stepsListDict,
                     [{'StepId': 123,
                       'StepName': 'Stripping14-Stripping',
                       'ApplicationName': 'DaVinci',
                       'ApplicationVersion': 'v2r2',
                       'ExtraOptions': '',
                       'ProcessingPass': 'procPass',
                       'OptionsFormat': '',
                       'SystemConfig': '',
                       'OptionFiles': 'optsFiles;$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py',
                       'Visible': 'Yes',
                       'ExtraPackages': 'eps',
                       'prodStepID': "123['SDST']",
                       'mcTCK': '',
                       'visibilityFlag': [{'Visible': 'N',
                                           'FileType': 'BHADRON.DST'},
                                          {'Visible': 'N',
                                           'FileType': 'CALIBRATION.DST'}],
                         'DDDB': '',
                         'CONDDB': '123456',
                         'DQTag': '',
                         'isMulticore': 'N',
                         'fileTypesIn': ['SDST'],
                         'fileTypesOut':['BHADRON.DST',
                                         'CALIBRATION.DST']}])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.compressionLvl = ['HIGH']
    pr.outputVisFlag = [{'1': 'Y'}]
    # pr.outputVisFlag = [{'Visibility':'N','FileType':'BHADRON.DST'},{'Visibility':'N','FileType':'CALIBRATION.DST'}]
    pr.stepsList = ['999']
    pr.resolveSteps()
    self.assertEqual(pr.stepsListDict, [{'StepId': 999, 'StepName': 'Stripping28',
                                         'ApplicationName': 'DaVinci',
                                         'ApplicationVersion': 'v41r3',
                                         'ExtraOptions': '',
                                         # 'OptionFiles':'$APPCONFIGOPTS/Persistency/Compression-ZLIB-1.py;$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py',
                                         # 'OptionFiles':'$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py',
                                         'OptionFiles': '',
                                         'Visible': 'Yes', 'Usable': 'Yes', 'ProcessingPass': 'Stripping28',
                                         'SystemConfig': 'x86_64-slc5-gcc49-opt',
                                         'ExtraPackages': 'AppConfig.v3r306', 'mcTCK': '', 'prodStepID': "999['SDST']",
                                         'DDDB': 'dddb-20150724', 'CONDDB': 'cond-20161011', 'DQTag': '',
                                         'isMulticore': 'N', 'fileTypesIn': ['SDST'],
                                         'visibilityFlag': [{'Visible': 'Y', 'FileType': 'BHADRON.DST'},
                                                            {'Visible': 'Y', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST']}])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.compressionLvl = ['HIGH']
    pr.outputVisFlag = [{'1': 'Y'}]
    pr.stepsList = ['998']
    pr.resolveSteps()
    self.assertEqual(pr.stepsListDict, [{'StepId': 998, 'StepName': 'Stripping28',
                                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v41r3',
                                         'ExtraOptions': '',
                                         # 'OptionFiles':'$APPCONFIGOPTS/Persistency/Compression-ZLIB-1.py;$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py',
                                         'OptionFiles': '$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py',
                                         'Visible': 'Yes', 'Usable': 'Yes', 'ProcessingPass': 'Stripping28',
                                         'SystemConfig': 'x86_64-slc6-gcc49-opt',
                                         'ExtraPackages': 'AppConfig.v3r306', 'mcTCK': '', 'prodStepID': "998['SDST']",
                                         'DDDB': 'dddb-20150724', 'CONDDB': 'cond-20161011',
                                         'DQTag': '', 'isMulticore': 'N', 'fileTypesIn': ['SDST'],
                                         'visibilityFlag':[{'Visible': 'Y', 'FileType': 'BHADRON.DST'},
                                                           {'Visible': 'Y', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST']}])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.stepsList = ['999']
    pr.outputVisFlag = [{'1': 'Y'}]
    pr.compressionLvl = [''] * 4
    pr.resolveSteps()
    self.assertEqual(pr.stepsListDict, [{'StepId': 999, 'StepName': 'Stripping28',
                                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v41r3',
                                         'ExtraOptions': '',
                                         # 'OptionFiles':'$APPCONFIGOPTS/Persistency/Compression-ZLIB-1.py;$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py',
                                         'OptionFiles': '',
                                         'Visible': 'Yes', 'Usable': 'Yes', 'ProcessingPass': 'Stripping28',
                                         'SystemConfig': 'x86_64-slc5-gcc49-opt',
                                         'ExtraPackages': 'AppConfig.v3r306', 'mcTCK': '', 'prodStepID': "999['SDST']",
                                         'DDDB': 'dddb-20150724', 'CONDDB': 'cond-20161011', 'DQTag': '',
                                         'isMulticore': 'N', 'fileTypesIn': ['SDST'],
                                         'visibilityFlag':[{'Visible': 'Y', 'FileType': 'BHADRON.DST'},
                                                           {'Visible': 'Y', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST']}])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.compressionLvl = ['HIGH']
    pr.outputVisFlag = [{'1': 'Y'}]
    pr.stepsList = ['997']
    pr.resolveSteps()
    optionFilesStep997 = '$APPCONFIGOPTS/Brunel/DataType-2016.py;'
    optionFilesStep997 += '$APPCONFIGOPTS/Brunel/PbPb-GECs.py;$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py'
    self.assertEqual(pr.stepsListDict, [{'StepId': 997, 'StepName': 'Stripping28',
                                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v41r3',
                                         'ExtraOptions': '',
                                         'OptionFiles': optionFilesStep997,
                                         'Visible': 'Yes', 'Usable': 'Yes', 'ProcessingPass': 'Stripping28',
                                         'SystemConfig': 'x86_64-slc6-gcc49-opt',
                                         'ExtraPackages': 'AppConfig.v3r306', 'mcTCK': '', 'prodStepID': "997['SDST']",
                                         'DDDB': 'dddb-20150724', 'CONDDB': 'cond-20161011', 'DQTag': '',
                                         'isMulticore': 'N', 'fileTypesIn': ['SDST'],
                                         'visibilityFlag':[{'Visible': 'Y', 'FileType': 'BHADRON.DST'},
                                                           {'Visible': 'Y', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST']}])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.compressionLvl = ['HIGH']
    pr.outputVisFlag = [{'1': 'Y'}]
    pr.stepsList = ['996']
    pr.resolveSteps()
    optionFilesStep996 = '$APPCONFIGOPTS/Brunel/DataType-2016.py;'
    optionFilesStep996 += '$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py;$APPCONFIGOPTS/Brunel/PbPb-GECs.py'
    self.assertEqual(pr.stepsListDict, [{'StepId': 996, 'StepName': 'Stripping28',
                                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v41r3',
                                         'ExtraOptions': '',
                                         'OptionFiles': optionFilesStep996,
                                         'Visible': 'Yes', 'Usable': 'Yes', 'ProcessingPass': 'Stripping28',
                                         'SystemConfig': 'x86_64-slc6-gcc49-opt',
                                         'ExtraPackages': 'AppConfig.v3r306', 'mcTCK': '',
                                         'prodStepID': "996['SDST']",
                                         'DDDB': 'dddb-20150724', 'CONDDB': 'cond-20161011', 'DQTag': '',
                                         'isMulticore': 'N', 'fileTypesIn': ['SDST'],
                                         'visibilityFlag':[{'Visible': 'Y', 'FileType': 'BHADRON.DST'},
                                                           {'Visible': 'Y', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST']}])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.compressionLvl = [''] * 4
    pr.outputVisFlag = [{'1': 'Y'}]
    pr.stepsList = ['1098']
    pr.resolveSteps()
    self.assertEqual(pr.stepsListDict,
                     [{'StepId': 1098,
                       'StepName': 'Stripping28',
                       'ApplicationName': 'DaVinci',
                       'ApplicationVersion': 'v41r3',
                       'ExtraOptions': '',
                       'OptionFiles': '$APPCONFIGOPTS/Brunel/DataType-2016.py;',
                       'Visible': 'Yes',
                       'Usable': 'Yes',
                       'ProcessingPass': 'Stripping28',
                       'SystemConfig': 'x86_64-slc6-gcc49-opt',
                       'ExtraPackages': 'AppConfig.v3r306',
                       'mcTCK': '',
                       'prodStepID': "1098['SDST']",
                       'DDDB': 'dddb-20150724',
                       'CONDDB': 'cond-20161011',
                       'DQTag': '',
                       'isMulticore': 'N',
                       'fileTypesIn': ['SDST'],
                         'visibilityFlag':[{'Visible': 'Y',
                                            'FileType': 'BHADRON.DST'},
                                           {'Visible': 'Y',
                                            'FileType': 'CALIBRATION.DST'}],
                         'fileTypesOut': ['BHADRON.DST',
                                          'CALIBRATION.DST']}])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.compressionLvl = ['HIGH']
    pr.outputVisFlag = [{'1': 'Y'}]
    pr.specialOutputVisFlag = [{'1': {'BHADRON.DST': 'Y'}}]
    pr.stepsList = ['1098']
    pr.resolveSteps()
    optionFilesStep1098 = '$APPCONFIGOPTS/Brunel/DataType-2016.py;'
    optionFilesStep1098 += '$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py;'
    optionFilesStep1098 += '$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py'
    self.assertEqual(pr.stepsListDict, [{'StepId': 1098, 'StepName': 'Stripping28',
                                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v41r3',
                                         'ExtraOptions': '',
                                         'OptionFiles': optionFilesStep1098,
                                         'Visible': 'Yes', 'Usable': 'Yes', 'ProcessingPass': 'Stripping28',
                                         'SystemConfig': 'x86_64-slc6-gcc49-opt',
                                         'ExtraPackages': 'AppConfig.v3r306',
                                         'mcTCK': '', 'prodStepID': "1098['SDST']",
                                         'DDDB': 'dddb-20150724', 'CONDDB': 'cond-20161011', 'DQTag': '',
                                         'isMulticore': 'N', 'fileTypesIn': ['SDST'],
                                         'visibilityFlag':[{'Visible': 'Y', 'FileType': 'BHADRON.DST'},
                                                           {'Visible': 'Y', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST']}])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.compressionLvl = ['']
    pr.outputVisFlag = [{'1': 'Y'}]
    pr.specialOutputVisFlag = [{'1': {'BHADRON.DST': 'Y'}}]
    pr.stepsList = ['1099']
    pr.resolveSteps()
    self.assertEqual(pr.stepsListDict,
                     [{'StepId': 1099,
                       'StepName': 'Stripping28',
                       'ApplicationName': 'DaVinci',
                       'ApplicationVersion': 'v41r3',
                       'ExtraOptions': '',
                       'OptionFiles': '$APPCONFIGOPTS/Brunel/DataType-2016.py;',
                       'Visible': 'Yes',
                       'Usable': 'Yes',
                       'ProcessingPass': 'Stripping28',
                       'SystemConfig': 'x86_64-slc6-gcc49-opt',
                       'ExtraPackages': 'AppConfig.v3r306',
                       'mcTCK': '',
                       'prodStepID': "1099['SDST']",
                       'DDDB': 'dddb-20150724',
                       'CONDDB': 'cond-20161011',
                       'DQTag': '',
                       'isMulticore': 'N',
                       'fileTypesIn': ['SDST'],
                         'visibilityFlag':[{'Visible': 'Y',
                                            'FileType': 'BHADRON.DST'},
                                           {'Visible': 'Y',
                                            'FileType': 'CALIBRATION.DST'}],
                         'fileTypesOut': ['BHADRON.DST',
                                          'CALIBRATION.DST']}])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.compressionLvl = ['', 'HIGH', 'LOW']
    pr.outputVisFlag = [{'1': 'N', '2': 'N', '3': 'N'}]
    pr.specialOutputVisFlag = [{'3': {'BHADRON.DST': 'Y'}}]
    pr.stepsList = ['1098', '996', '997']
    pr.resolveSteps()
    optionFilesStep996 = '$APPCONFIGOPTS/Brunel/DataType-2016.py;'
    optionFilesStep996 += '$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py;'
    optionFilesStep996 += '$APPCONFIGOPTS/Brunel/PbPb-GECs.py'
    optionFilesStep997 = '$APPCONFIGOPTS/Brunel/DataType-2016.py;'
    optionFilesStep997 += '$APPCONFIGOPTS/Brunel/PbPb-GECs.py;'
    optionFilesStep997 += '$APPCONFIGOPTS/Persistency/Compression-ZLIB-1.py'
    self.assertEqual(pr.stepsListDict, [{'StepId': 1098, 'StepName': 'Stripping28',
                                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v41r3',
                                         'ExtraOptions': '',
                                         'OptionFiles': '$APPCONFIGOPTS/Brunel/DataType-2016.py;',
                                         'Visible': 'Yes', 'Usable': 'Yes', 'ProcessingPass': 'Stripping28',
                                         'SystemConfig': 'x86_64-slc6-gcc49-opt',
                                         'ExtraPackages': 'AppConfig.v3r306',
                                         'mcTCK': '', 'prodStepID': "1098['SDST']",
                                         'DDDB': 'dddb-20150724', 'CONDDB': 'cond-20161011', 'DQTag': '',
                                         'isMulticore': 'N', 'fileTypesIn': ['SDST'],
                                         'visibilityFlag':[{'Visible': 'N', 'FileType': 'BHADRON.DST'},
                                                           {'Visible': 'N', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST']},
                                        {'StepId': 996, 'StepName': 'Stripping28',
                                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v41r3',
                                         'ExtraOptions': '',
                                         'OptionFiles': optionFilesStep996,
                                         'Visible': 'Yes', 'Usable': 'Yes', 'ProcessingPass': 'Stripping28',
                                         'SystemConfig': 'x86_64-slc6-gcc49-opt',
                                         'ExtraPackages': 'AppConfig.v3r306', 'mcTCK': '', 'prodStepID': "996['SDST']",
                                         'DDDB': 'dddb-20150724', 'CONDDB': 'cond-20161011', 'DQTag': '',
                                         'isMulticore': 'N', 'fileTypesIn': ['SDST'],
                                         'visibilityFlag':[{'Visible': 'N', 'FileType': 'BHADRON.DST'},
                                                           {'Visible': 'N', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST']},
                                        {'StepId': 997, 'StepName': 'Stripping28',
                                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v41r3',
                                         'ExtraOptions': '',
                                         'OptionFiles': optionFilesStep997,
                                         'Visible': 'Yes', 'Usable': 'Yes', 'ProcessingPass': 'Stripping28',
                                         'SystemConfig': 'x86_64-slc6-gcc49-opt',
                                         'ExtraPackages': 'AppConfig.v3r306', 'mcTCK': '', 'prodStepID': "997['SDST']",
                                         'DDDB': 'dddb-20150724', 'CONDDB': 'cond-20161011', 'DQTag': '',
                                         'isMulticore': 'N', 'fileTypesIn': ['SDST'],
                                         'visibilityFlag':[{'Visible': 'Y', 'FileType': 'BHADRON.DST'},
                                                           {'Visible': 'N', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST']}
                                        ])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.compressionLvl = ['', 'HIGH']
    pr.outputVisFlag = [{'1': 'N', '2': 'N', '3': 'N'}]
    pr.specialOutputVisFlag = [{'2': {'BHADRON.DST': 'Y'}}]
    pr.stepsList = ['1098', '996', '997']
    pr.resolveSteps()
    optionFilesStep996 = '$APPCONFIGOPTS/Brunel/DataType-2016.py;'
    optionFilesStep996 += '$APPCONFIGOPTS/Persistency/Compression-LZMA-4.py;$APPCONFIGOPTS/Brunel/PbPb-GECs.py'
    optionFilesStep997 = '$APPCONFIGOPTS/Brunel/DataType-2016.py;$APPCONFIGOPTS/Brunel/PbPb-GECs.py'
    self.assertEqual(pr.stepsListDict, [{'StepId': 1098, 'StepName': 'Stripping28',
                                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v41r3',
                                         'ExtraOptions': '',
                                         'OptionFiles': '$APPCONFIGOPTS/Brunel/DataType-2016.py;',
                                         'Visible': 'Yes', 'Usable': 'Yes', 'ProcessingPass': 'Stripping28',
                                         'SystemConfig': 'x86_64-slc6-gcc49-opt',
                                         'ExtraPackages': 'AppConfig.v3r306',
                                         'mcTCK': '', 'prodStepID': "1098['SDST']",
                                         'DDDB': 'dddb-20150724', 'CONDDB': 'cond-20161011', 'DQTag': '',
                                         'isMulticore': 'N', 'fileTypesIn': ['SDST'],
                                         'visibilityFlag':[{'Visible': 'N', 'FileType': 'BHADRON.DST'},
                                                           {'Visible': 'N', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST']},
                                        {'StepId': 996, 'StepName': 'Stripping28',
                                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v41r3',
                                         'ExtraOptions': '',
                                         'OptionFiles': optionFilesStep996,
                                         'Visible': 'Yes', 'Usable': 'Yes', 'ProcessingPass': 'Stripping28',
                                         'SystemConfig': 'x86_64-slc6-gcc49-opt',
                                         'ExtraPackages': 'AppConfig.v3r306',
                                         'mcTCK': '', 'prodStepID': "996['SDST']",
                                         'DDDB': 'dddb-20150724', 'CONDDB': 'cond-20161011', 'DQTag': '',
                                         'isMulticore': 'N', 'fileTypesIn': ['SDST'],
                                         'visibilityFlag':[{'Visible': 'Y', 'FileType': 'BHADRON.DST'},
                                                           {'Visible': 'N', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST']},
                                        {'StepId': 997, 'StepName': 'Stripping28',
                                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v41r3',
                                         'ExtraOptions': '',
                                         'OptionFiles': optionFilesStep997,
                                         'Visible': 'Yes', 'Usable': 'Yes', 'ProcessingPass': 'Stripping28',
                                         'SystemConfig': 'x86_64-slc6-gcc49-opt',
                                         'ExtraPackages': 'AppConfig.v3r306',
                                         'mcTCK': '', 'prodStepID': "997['SDST']",
                                         'DDDB': 'dddb-20150724', 'CONDDB': 'cond-20161011', 'DQTag': '',
                                         'isMulticore': 'N', 'fileTypesIn': ['SDST'],
                                         'visibilityFlag':[{'Visible': 'N', 'FileType': 'BHADRON.DST'},
                                                           {'Visible': 'N', 'FileType': 'CALIBRATION.DST'}],
                                         'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST']}
                                        ])

  def test__determineOutputSEs(self):
    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.stepsList = ['123', '456']
    pr.compressionLvl = ['LOW', 'HIGH']
    pr.outputVisFlag = [{'1': 'N'}, {'2': 'N'}]
    pr.resolveSteps()
    pr.outputSEs = ['SE1', 'SE2']
    pr.specialOutputSEs = [{}, {}]
    pr._determineOutputSEs()
    self.assertEqual(pr.outputSEsPerFileType, [{'CALIBRATION.DST': 'SE1', 'BHADRON.DST': 'SE1'},
                                               {'CALIBRATION.DST': 'SE2', 'BHADRON.DST': 'SE2'}])

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.outputVisFlag = [{'1': 'N'}, {'2': 'N'}]
    pr.stepsList = ['123', '456']
    pr.compressionLvl = ['LOW', 'HIGH']
    pr.resolveSteps()
    pr.outputSEs = ['SE1', 'SE2']
    pr.specialOutputSEs = [{'CALIBRATION.DST': 'SE3'}, {}]
    pr._determineOutputSEs()
    self.assertEqual(pr.outputSEsPerFileType, [{'CALIBRATION.DST': 'SE3', 'BHADRON.DST': 'SE1'},
                                               {'CALIBRATION.DST': 'SE2', 'BHADRON.DST': 'SE2'}])

  def test__applyOptionalCorrections(self):

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.prodsTypeList = ['DataStripping', 'Merge']
    pr.plugins = ['ByRun', 'BySize']
    pr.stepsListDict = [stepStripp, mergeStep]
    pr.outputVisFlag = [{'1': 'N'}, {'2': 'N'}]
    pr.stepsList = [123, 456]
    pr.stepsInProds = [[1], [2]]
    pr.outputSEs = ['Tier1-DST', 'Tier1-M-DST']
    pr.specialOutputSEs = [{}, {}]
    pr.priorities = [1, 4]
    pr.cpus = [10, 100]
    pr.groupSizes = [1, 2]
    pr.bkFileType = 'TYPE'
    pr.removeInputsFlags = [False, True]
    pr.outputFileMasks = ['', 'dst']
    pr.outputFileSteps = ['', '2']
    pr.inputs = [['aa'], []]
    pr.inputDataPolicies = ['dl', 'pr']
    pr.events = [-1, -1]
    pr.bkQueries = ['Full', 'fromPreviousProd']
    pr.targets = ['Target1', 'Target2']
    pr.multicore = ['False', 'True']
    pr.processors = [(2, 4), (0, 0)]
    pr.outputModes = ['Local', 'Any']
    pr.ancestorDepths = [0, 1]
    pr._applyOptionalCorrections()
    prodsTypeListExpected = ['DataStripping', 'Merge', 'Merge', 'Merge']
    pluginsExpected = ['ByRun', 'BySize', 'BySize', 'BySize']
    stepsListDictExpected = [stepStripp, mergeStepBHADRON, mergeStepCALIBRA, mergeStepPIDMDST]
    stepsInProdExpected = [[1], [2], [3], [4]]
    outputSEsExpected = ['Tier1-DST', 'Tier1-M-DST', 'Tier1-M-DST', 'Tier1-M-DST']
    specialOutputSEsExpected = [{}, {}, {}, {}]
    prioritiesExpected = [1, 4, 4, 4]
    cpusExpected = [10, 100, 100, 100]
    bkQueriesExpected = ['Full', 'fromPreviousProd', 'fromPreviousProd', 'fromPreviousProd']
    groupSizesExpected = [1, 2, 2, 2]
    previousProdsExpected = [None, 1, 1, 1]
    removeInputsFlagExpected = [False, True, True, True]
    outputFileMasksExpected = ['', 'dst', 'dst', 'dst']
    outputFileStepsExpected = ['', '2', '2', '2']
    inputsExpected = [['aa'], [], [], []]
    inputDataPoliciesExpected = ['dl', 'pr', 'pr', 'pr']
    eventsExpected = [-1, -1, -1, -1]
    targetsExpected = ['Target1', 'Target2', 'Target2', 'Target2']
    multicoreExpected = ['False', 'True', 'True', 'True']
    processorsExpected = [(2, 4), (0, 0), (0, 0), (0, 0)]
    outputModeExpected = ['Local', 'Any', 'Any', 'Any']
    ancestorDepthsExpected = [0, 1, 1, 1]
    self.assertEqual(pr.prodsTypeList, prodsTypeListExpected)
    self.assertEqual(pr.plugins, pluginsExpected)
    self.assertEqual(pr.stepsListDict, stepsListDictExpected)
    self.assertEqual(pr.stepsInProds, stepsInProdExpected)
    self.assertEqual(pr.outputSEs, outputSEsExpected)
    self.assertEqual(pr.specialOutputSEs, specialOutputSEsExpected)
    self.assertEqual(pr.priorities, prioritiesExpected)
    self.assertEqual(pr.cpus, cpusExpected)
    self.assertEqual(pr.groupSizes, groupSizesExpected)
    self.assertEqual(pr.bkQueries, bkQueriesExpected)
    self.assertEqual(pr.previousProds, previousProdsExpected)
    self.assertEqual(pr.removeInputsFlags, removeInputsFlagExpected)
    self.assertEqual(pr.outputFileMasks, outputFileMasksExpected)
    self.assertEqual(pr.outputFileSteps, outputFileStepsExpected)
    self.assertEqual(pr.inputs, inputsExpected)
    self.assertEqual(pr.inputDataPolicies, inputDataPoliciesExpected)
    self.assertEqual(pr.events, eventsExpected)
    self.assertEqual(pr.targets, targetsExpected)
    self.assertEqual(pr.multicore, multicoreExpected)
    self.assertEqual(pr.processors, processorsExpected)
    self.assertEqual(pr.outputModes, outputModeExpected)
    self.assertEqual(pr.ancestorDepths, ancestorDepthsExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.prodsTypeList = ['DataStripping', 'Merge']
    pr.plugins = ['ByRun', 'ByRunFileTypeSizeWithFlush']
    pr.stepsListDict = [stepStripp, mergeStep]
    pr.stepsList = [123, 456]
    pr.stepsInProds = [[1], [2]]
    pr.outputSEs = ['Tier1-DST', 'Tier1-M-DST']
    pr.specialOutputSEs = [{'Type1': 'SE1'}, {}]
    pr.priorities = [1, 4]
    pr.cpus = [10, 100]
    pr.groupSizes = [1, 2]
    pr.bkFileType = ['Type1', 'Type2']
    pr.inputs = [[], ['aa']]
    pr.removeInputsFlags = [False, True]
    pr.outputFileMasks = ['', 'dst']
    pr.outputFileSteps = ['', '2']
    pr.inputDataPolicies = ['pr', 'dl']
    pr.events = [-1, -1]
    pr.bkQueries = ['Full', 'fromPreviousProd']
    pr.targets = ['Target1', 'Target2']
    pr.multicore = ['False', 'True']
    pr.processors = [2, 0]
    pr.outputModes = ['Local', 'Any']
    pr.ancestorDepths = [0, 0]
    pr._applyOptionalCorrections()
    prodsTypeListExpected = ['DataStripping', 'Merge']
    pluginsExpected = ['ByRun', 'ByRunFileTypeSizeWithFlush']
    stepsListDictExpected = [stepStripp, mergeStep]
    stepsInProdExpected = [[1], [2]]
    outputSEsExpected = ['Tier1-DST', 'Tier1-M-DST']
    specialOutputSEsExpected = [{'Type1': 'SE1'}, {}]
    prioritiesExpected = [1, 4]
    cpusExpected = [10, 100]
    groupSizesExpected = [1, 2]
    bkQueriesExpected = ['Full', 'fromPreviousProd']
    previousProdsExpected = [None, 1]
    removeInputsFlagExpected = [False, True]
    outputFileMasksExpected = ['', 'dst']
    outputFileStepsExpected = ['', '2']
    inputsExpected = [[], ['aa']]
    inputDataPoliciesExpected = ['pr', 'dl']
    eventsExpected = [-1, -1]
    targetsExpected = ['Target1', 'Target2']
    multicoreExpected = ['False', 'True']
    processorsExpected = [2, 0]
    outputModeExpected = ['Local', 'Any']
    ancestorDepthsExpected = [0, 0]
    self.assertEqual(pr.prodsTypeList, prodsTypeListExpected)
    self.assertEqual(pr.plugins, pluginsExpected)
    self.assertEqual(pr.stepsListDict, stepsListDictExpected)
    self.assertEqual(pr.stepsInProds, stepsInProdExpected)
    self.assertEqual(pr.outputSEs, outputSEsExpected)
    self.assertEqual(pr.specialOutputSEs, specialOutputSEsExpected)
    self.assertEqual(pr.priorities, prioritiesExpected)
    self.assertEqual(pr.cpus, cpusExpected)
    self.assertEqual(pr.groupSizes, groupSizesExpected)
    self.assertEqual(pr.bkQueries, bkQueriesExpected)
    self.assertEqual(pr.previousProds, previousProdsExpected)
    self.assertEqual(pr.removeInputsFlags, removeInputsFlagExpected)
    self.assertEqual(pr.outputFileMasks, outputFileMasksExpected)
    self.assertEqual(pr.outputFileSteps, outputFileStepsExpected)
    self.assertEqual(pr.inputs, inputsExpected)
    self.assertEqual(pr.inputDataPolicies, inputDataPoliciesExpected)
    self.assertEqual(pr.events, eventsExpected)
    self.assertEqual(pr.targets, targetsExpected)
    self.assertEqual(pr.multicore, multicoreExpected)
    self.assertEqual(pr.processors, processorsExpected)
    self.assertEqual(pr.outputModes, outputModeExpected)
    self.assertEqual(pr.ancestorDepths, ancestorDepthsExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.prodsTypeList = ['Merge', 'Merge']
    pr.plugins = ['BySize', 'ByRunFileTypeSizeWithFlush']
    pr.stepsListDict = [mergeStep, mergeStep]
    pr.stepsList = [456, 456]
    pr.stepsInProds = [[1], [2]]
    pr.outputSEs = ['Tier1-DST', 'Tier1-M-DST']
    pr.specialOutputSEs = [{'Type1': 'SE1'}, {}]
    pr.priorities = [1, 4]
    pr.cpus = [10, 100]
    pr.groupSizes = [1, 2]
    pr.removeInputsFlags = [True, True]
    pr.outputFileMasks = ['', 'dst']
    pr.outputFileSteps = ['', '2']
    pr.inputs = [[], ['aa']]
    pr.inputDataPolicies = ['dl', 'pr']
    pr.events = [-1, -1]
    pr.bkQueries = ['fromPreviousProd', 'fromPreviousProd']
    pr.targets = ['Target1', 'Target2']
    pr.multicore = ['False', 'True']
    pr.processors = [2, 0]
    pr.outputModes = ['Local', 'Any']
    pr.ancestorDepths = [1, 0]
    pr._applyOptionalCorrections()
    prodsTypeListExpected = ['Merge', 'Merge', 'Merge', 'Merge']
    pluginsExpected = ['BySize', 'BySize', 'BySize', 'ByRunFileTypeSizeWithFlush']
    stepsListDictExpected = [mergeStepBHADRON, mergeStepCALIBRA, mergeStepPIDMDST, mergeStep]
    stepsInProdExpected = [[1], [2], [3], [4]]
    outputSEsExpected = ['Tier1-DST', 'Tier1-DST', 'Tier1-DST', 'Tier1-M-DST']
    specialOutputSEsExpected = [{'Type1': 'SE1'}, {'Type1': 'SE1'}, {'Type1': 'SE1'}, {}]
    prioritiesExpected = [1, 1, 1, 4]
    cpusExpected = [10, 10, 10, 100]
    groupSizesExpected = [1, 1, 1, 2]
    bkQueriesExpected = ['fromPreviousProd', 'fromPreviousProd', 'fromPreviousProd', 'fromPreviousProd']
    previousProdsExpected = [None, None, None, 1]
    removeInputsFlagExpected = [True, True, True, True]
    outputFileMasksExpected = ['', '', '', 'dst']
    outputFileStepsExpected = ['', '', '', '2']
    inputsExpected = [[], [], [], ['aa']]
    inputDataPoliciesExpected = ['dl', 'dl', 'dl', 'pr']
    eventsExpected = [-1, -1, -1, -1]
    targetsExpected = ['Target1', 'Target1', 'Target1', 'Target2']
    multicoreExpected = ['False', 'False', 'False', 'True']
    processorsExpected = [2, 2, 2, 0]
    outputModeExpected = ['Local', 'Local', 'Local', 'Any']
    ancestorDepthsExpected = [1, 1, 1, 0]
    self.assertEqual(pr.prodsTypeList, prodsTypeListExpected)
    self.assertEqual(pr.plugins, pluginsExpected)
    self.assertEqual(pr.stepsListDict, stepsListDictExpected)
    self.assertEqual(pr.stepsInProds, stepsInProdExpected)
    self.assertEqual(pr.outputSEs, outputSEsExpected)
    self.assertEqual(pr.specialOutputSEs, specialOutputSEsExpected)
    self.assertEqual(pr.priorities, prioritiesExpected)
    self.assertEqual(pr.cpus, cpusExpected)
    self.assertEqual(pr.groupSizes, groupSizesExpected)
    self.assertEqual(pr.bkQueries, bkQueriesExpected)
    self.assertEqual(pr.previousProds, previousProdsExpected)
    self.assertEqual(pr.removeInputsFlags, removeInputsFlagExpected)
    self.assertEqual(pr.outputFileMasks, outputFileMasksExpected)
    self.assertEqual(pr.outputFileSteps, outputFileStepsExpected)
    self.assertEqual(pr.inputs, inputsExpected)
    self.assertEqual(pr.inputDataPolicies, inputDataPoliciesExpected)
    self.assertEqual(pr.events, eventsExpected)
    self.assertEqual(pr.targets, targetsExpected)
    self.assertEqual(pr.multicore, multicoreExpected)
    self.assertEqual(pr.processors, processorsExpected)
    self.assertEqual(pr.outputModes, outputModeExpected)
    self.assertEqual(pr.ancestorDepths, ancestorDepthsExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.prodsTypeList = ['DataStripping']
    pr.plugins = ['ByRun']
    pr.stepsListDict = [stepStripp, mergeStep]
    pr.stepsList = [123]
    pr.stepsInProds = [[1, 2]]
    pr.priorities = [1]
    pr.outputSEs = ['Tier1-DST']
    pr.cpus = [10]
    pr.groupSizes = [1]
    pr.bkFileType = 'TYPE'
    pr.removeInputsFlags = [False]
    pr.outputFileMasks = ['']
    pr.outputFileSteps = ['2']
    pr.inputs = [[]]
    pr.inputDataPolicies = ['']
    pr.events = [-1]
    pr.bkQueries = ['Full']
    pr.targets = ['Target1']
    pr.multicore = ['False']
    pr.processors = [2]
    pr.outputModes = ['Local']
    pr.ancestorDepths = [1]
    pr._applyOptionalCorrections()
    prodsTypeListExpected = ['DataStripping']
    pluginsExpected = ['ByRun']
    stepsListDictExpected = [stepStripp, mergeStep]
    stepsInProdExpected = [[1, 2]]
    outputSEsExpected = ['Tier1-DST']
    prioritiesExpected = [1]
    cpusExpected = [10]
    groupSizesExpected = [1]
    bkQueriesExpected = ['Full']
    previousProdsExpected = [None]
    removeInputsFlagExpected = [False]
    outputFileMasksExpected = ['']
    outputFileStepsExpected = ['2']
    inputsExpected = [[]]
    inputDataPoliciesExpected = ['']
    eventsExpected = [-1]
    targetsExpected = ['Target1']
    multicoreExpected = ['False']
    processorsExpected = [2]
    outputModeExpected = ['Local']
    ancestorDepthsExpected = [1]
    self.assertEqual(pr.prodsTypeList, prodsTypeListExpected)
    self.assertEqual(pr.plugins, pluginsExpected)
    self.assertEqual(pr.stepsListDict, stepsListDictExpected)
    self.assertEqual(pr.stepsInProds, stepsInProdExpected)
    self.assertEqual(pr.outputSEs, outputSEsExpected)
    self.assertEqual(pr.priorities, prioritiesExpected)
    self.assertEqual(pr.cpus, cpusExpected)
    self.assertEqual(pr.groupSizes, groupSizesExpected)
    self.assertEqual(pr.bkQueries, bkQueriesExpected)
    self.assertEqual(pr.previousProds, previousProdsExpected)
    self.assertEqual(pr.removeInputsFlags, removeInputsFlagExpected)
    self.assertEqual(pr.outputFileMasks, outputFileMasksExpected)
    self.assertEqual(pr.outputFileSteps, outputFileStepsExpected)
    self.assertEqual(pr.inputs, inputsExpected)
    self.assertEqual(pr.inputDataPolicies, inputDataPoliciesExpected)
    self.assertEqual(pr.events, eventsExpected)
    self.assertEqual(pr.targets, targetsExpected)
    self.assertEqual(pr.multicore, multicoreExpected)
    self.assertEqual(pr.processors, processorsExpected)
    self.assertEqual(pr.outputModes, outputModeExpected)
    self.assertEqual(pr.ancestorDepths, ancestorDepthsExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.prodsTypeList = ['DataStripping', 'Merge', 'Merge']
    pr.plugins = ['ByRun', 'BySize', 'ByRunFileTypeSizeWithFlush']
    pr.stepsListDict = [stepStripp, mergeStep, mergeStep]
    pr.stepsList = [123, 456, 456]
    pr.stepsInProds = [[1], [2], [3]]
    pr.outputSEs = ['Tier1-DST', 'Tier1-M-DST', 'Tier1-M-DST']
    pr.priorities = [1, 4, 5]
    pr.cpus = [10, 100, 1000]
    pr.groupSizes = [1, 2, 3]
    pr.bkFileType = 'TYPE'
    pr.removeInputsFlags = [False, True, True]
    pr.outputFileMasks = ['', 'dst', '']
    pr.outputFileSteps = ['', '2', '']
    pr.inputs = [[], ['aa'], ['bb']]
    pr.inputDataPolicies = ['', 'dl', 'pr']
    pr.bkQueries = ['Full', 'fromPreviousProd', 'fromPreviousProd']
    pr.events = [-1, -1, -1]
    pr.targets = ['Target1', 'Target2', 'Target3']
    pr.multicore = ['False', 'True', 'False']
    pr.processors = [2, 0, 4]
    pr.outputModes = ['Local', 'Any', 'Any']
    pr.ancestorDepths = [1, 2, 3]
    pr._applyOptionalCorrections()
    prodsTypeListExpected = ['DataStripping', 'Merge', 'Merge', 'Merge', 'Merge']
    pluginsExpected = ['ByRun', 'BySize', 'BySize', 'BySize', 'ByRunFileTypeSizeWithFlush']
    stepsListDictExpected = [stepStripp, mergeStepBHADRON, mergeStepCALIBRA, mergeStepPIDMDST, mergeStep]
    stepsInProdExpected = [[1], [2], [3], [4], [5]]
    outputSEsExpected = ['Tier1-DST', 'Tier1-M-DST', 'Tier1-M-DST', 'Tier1-M-DST', 'Tier1-M-DST']
    prioritiesExpected = [1, 4, 4, 4, 5]
    groupSizesExpected = [1, 2, 2, 2, 3]
    cpusExpected = [10, 100, 100, 100, 1000]
    bkQueriesExpected = ['Full', 'fromPreviousProd', 'fromPreviousProd', 'fromPreviousProd', 'fromPreviousProd']
    previousProdsExpected = [None, 1, 1, 1, 2]
    removeInputsFlagExpected = [False, True, True, True, True]
    outputFileMasksExpected = ['', 'dst', 'dst', 'dst', '']
    outputFileStepsExpected = ['', '2', '2', '2', '']
    inputsExpected = [[], ['aa'], ['aa'], ['aa'], ['bb']]
    inputDataPoliciesExpected = ['', 'dl', 'dl', 'dl', 'pr']
    eventsExpected = [-1, -1, -1, -1, -1]
    targetsExpected = ['Target1', 'Target2', 'Target2', 'Target2', 'Target3']
    multicoreExpected = ['False', 'True', 'True', 'True', 'False']
    processorsExpected = [2, 0, 0, 0, 4]
    outputModeExpected = ['Local', 'Any', 'Any', 'Any', 'Any']
    ancestorDepthsExpected = [1, 2, 2, 2, 3]
    self.assertEqual(pr.prodsTypeList, prodsTypeListExpected)
    self.assertEqual(pr.plugins, pluginsExpected)
    self.assertEqual(pr.stepsListDict, stepsListDictExpected)
    self.assertEqual(pr.stepsInProds, stepsInProdExpected)
    self.assertEqual(pr.outputSEs, outputSEsExpected)
    self.assertEqual(pr.priorities, prioritiesExpected)
    self.assertEqual(pr.cpus, cpusExpected)
    self.assertEqual(pr.groupSizes, groupSizesExpected)
    self.assertEqual(pr.bkQueries, bkQueriesExpected)
    self.assertEqual(pr.previousProds, previousProdsExpected)
    self.assertEqual(pr.removeInputsFlags, removeInputsFlagExpected)
    self.assertEqual(pr.outputFileMasks, outputFileMasksExpected)
    self.assertEqual(pr.outputFileSteps, outputFileStepsExpected)
    self.assertEqual(pr.inputs, inputsExpected)
    self.assertEqual(pr.inputDataPolicies, inputDataPoliciesExpected)
    self.assertEqual(pr.events, eventsExpected)
    self.assertEqual(pr.targets, targetsExpected)
    self.assertEqual(pr.multicore, multicoreExpected)
    self.assertEqual(pr.processors, processorsExpected)
    self.assertEqual(pr.outputModes, outputModeExpected)
    self.assertEqual(pr.ancestorDepths, ancestorDepthsExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.prodsTypeList = ['DataStripping', 'Merge']
    pr.plugins = ['ByRun', 'BySize']
    pr.stepsListDict = [stepStripp, mergeStep]
    pr.stepsList = [123, 456]
    pr.stepsInProds = [[3], [4]]
    pr.outputSEs = ['Tier1-DST', 'Tier1-M-DST']
    pr.priorities = [1, 4]
    pr.cpus = [10, 100]
    pr.groupSizes = [1, 2]
    pr.bkFileType = 'TYPE'
    pr.removeInputsFlags = [False, True]
    pr.outputFileMasks = ['', 'dst']
    pr.outputFileSteps = ['', '2']
    pr.inputs = [[], []]
    pr.inputDataPolicies = ['', 'dl']
    pr.events = [-1, -1]
    pr.bkQueries = ['Full', 'fromPreviousProd']
    pr.targets = ['Target1', 'Target2']
    pr.multicore = ['False', 'True']
    pr.processors = [2, 0]
    pr.outputModes = ['Local', 'Any']
    pr.ancestorDepths = [1, 0]
    pr._applyOptionalCorrections()
    prodsTypeListExpected = ['DataStripping', 'Merge', 'Merge', 'Merge']
    pluginsExpected = ['ByRun', 'BySize', 'BySize', 'BySize']
    stepsListDictExpected = [stepStripp, mergeStepBHADRON, mergeStepCALIBRA, mergeStepPIDMDST]
    stepsInProdExpected = [[3], [4], [5], [6]]
    outputSEsExpected = ['Tier1-DST', 'Tier1-M-DST', 'Tier1-M-DST', 'Tier1-M-DST']
    prioritiesExpected = [1, 4, 4, 4]
    cpusExpected = [10, 100, 100, 100]
    groupSizesExpected = [1, 2, 2, 2]
    bkQueriesExpected = ['Full', 'fromPreviousProd', 'fromPreviousProd', 'fromPreviousProd']
    previousProdsExpected = [None, 1, 1, 1]
    removeInputsFlagExpected = [False, True, True, True]
    outputFileMasksExpected = ['', 'dst', 'dst', 'dst']
    outputFileStepsExpected = ['', '2', '2', '2']
    inputsExpected = [[], [], [], []]
    inputDataPoliciesExpected = ['', 'dl', 'dl', 'dl']
    eventsExpected = [-1, -1, -1, -1]
    targetsExpected = ['Target1', 'Target2', 'Target2', 'Target2']
    multicoreExpected = ['False', 'True', 'True', 'True']
    processorsExpected = [2, 0, 0, 0]
    outputModeExpected = ['Local', 'Any', 'Any', 'Any']
    ancestorDepthsExpected = [1, 0, 0, 0]
    self.assertEqual(pr.prodsTypeList, prodsTypeListExpected)
    self.assertEqual(pr.plugins, pluginsExpected)
    self.assertEqual(pr.stepsListDict, stepsListDictExpected)
    self.assertEqual(pr.stepsInProds, stepsInProdExpected)
    self.assertEqual(pr.outputSEs, outputSEsExpected)
    self.assertEqual(pr.priorities, prioritiesExpected)
    self.assertEqual(pr.cpus, cpusExpected)
    self.assertEqual(pr.groupSizes, groupSizesExpected)
    self.assertEqual(pr.bkQueries, bkQueriesExpected)
    self.assertEqual(pr.previousProds, previousProdsExpected)
    self.assertEqual(pr.removeInputsFlags, removeInputsFlagExpected)
    self.assertEqual(pr.outputFileMasks, outputFileMasksExpected)
    self.assertEqual(pr.outputFileSteps, outputFileStepsExpected)
    self.assertEqual(pr.inputs, inputsExpected)
    self.assertEqual(pr.inputDataPolicies, inputDataPoliciesExpected)
    self.assertEqual(pr.events, eventsExpected)
    self.assertEqual(pr.targets, targetsExpected)
    self.assertEqual(pr.multicore, multicoreExpected)
    self.assertEqual(pr.processors, processorsExpected)
    self.assertEqual(pr.outputModes, outputModeExpected)
    self.assertEqual(pr.ancestorDepths, ancestorDepthsExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.prodsTypeList = ['DataStripping']
    pr.plugins = ['ByRun']
    pr.stepsListDict = [stepStripp, mergeStep]
    pr.stepsList = [123, 456]
    pr.stepsInProds = [[3, 4]]
    pr.priorities = [1]
    pr.outputSEs = ['Tier1-DST']
    pr.cpus = [10]
    pr.groupSizes = [1]
    pr.removeInputsFlags = [False]
    pr.outputFileMasks = ['dst']
    pr.outputFileSteps = ['2']
    pr.bkFileType = 'TYPE'
    pr.inputs = [[]]
    pr.inputDataPolicies = ['dl']
    pr.events = [-1]
    pr.bkQueries = ['Full']
    pr.targets = ['Target1']
    pr.multicore = ['False']
    pr.processors = [2]
    pr.outputModes = ['Local']
    pr.ancestorDepths = [1]
    pr._applyOptionalCorrections()
    prodsTypeListExpected = ['DataStripping']
    pluginsExpected = ['ByRun']
    stepsListDictExpected = [stepStripp, mergeStep]
    stepsInProdExpected = [[3, 4]]
    outputSEsExpected = ['Tier1-DST']
    prioritiesExpected = [1]
    cpusExpected = [10]
    groupSizesExpected = [1]
    bkQueriesExpected = ['Full']
    previousProdsExpected = [None]
    removeInputsFlagExpected = [False]
    outputFileMasksExpected = ['dst']
    outputFileStepsExpected = ['2']
    inputsExpected = [[]]
    inputDataPoliciesExpected = ['dl']
    eventsExpected = [-1]
    targetsExpected = ['Target1']
    multicoreExpected = ['False']
    processorsExpected = [2]
    outputModeExpected = ['Local']
    ancestorDepthsExpected = [1]
    self.assertEqual(pr.prodsTypeList, prodsTypeListExpected)
    self.assertEqual(pr.plugins, pluginsExpected)
    self.assertEqual(pr.stepsListDict, stepsListDictExpected)
    self.assertEqual(pr.stepsInProds, stepsInProdExpected)
    self.assertEqual(pr.outputSEs, outputSEsExpected)
    self.assertEqual(pr.priorities, prioritiesExpected)
    self.assertEqual(pr.cpus, cpusExpected)
    self.assertEqual(pr.groupSizes, groupSizesExpected)
    self.assertEqual(pr.bkQueries, bkQueriesExpected)
    self.assertEqual(pr.previousProds, previousProdsExpected)
    self.assertEqual(pr.removeInputsFlags, removeInputsFlagExpected)
    self.assertEqual(pr.outputFileMasks, outputFileMasksExpected)
    self.assertEqual(pr.outputFileSteps, outputFileStepsExpected)
    self.assertEqual(pr.inputs, inputsExpected)
    self.assertEqual(pr.inputDataPolicies, inputDataPoliciesExpected)
    self.assertEqual(pr.events, eventsExpected)
    self.assertEqual(pr.targets, targetsExpected)
    self.assertEqual(pr.multicore, multicoreExpected)
    self.assertEqual(pr.processors, processorsExpected)
    self.assertEqual(pr.outputModes, outputModeExpected)
    self.assertEqual(pr.ancestorDepths, ancestorDepthsExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.prodsTypeList = ['Merge', 'Merge']
    pr.plugins = ['BySize', 'ByRunFileTypeSizeWithFlush']
    pr.stepsListDict = [mergeStep, mergeStep]
    pr.stepsList = [456, 456]
    pr.stepsInProds = [[3], [4]]
    pr.outputSEs = ['Tier1-DST', 'Tier1-M-DST']
    pr.priorities = [1, 4]
    pr.cpus = [10, 100]
    pr.groupSizes = [1, 2]
    pr.removeInputsFlags = [True, True]
    pr.outputFileMasks = ['', 'dst']
    pr.outputFileSteps = ['', '2']
    pr.inputs = [['aa'], ['bb']]
    pr.inputDataPolicies = ['dl', 'pr']
    pr.events = [-1, -1]
    pr.bkQueries = ['Full', 'fromPreviousProd']
    pr.targets = ['Target1', 'Target2']
    pr.multicore = ['False', 'True']
    pr.processors = [2, 0]
    pr.outputModes = ['Local', 'Any']
    pr.ancestorDepths = [1, 0]
    pr._applyOptionalCorrections()
    prodsTypeListExpected = ['Merge', 'Merge', 'Merge', 'Merge']
    pluginsExpected = ['BySize', 'BySize', 'BySize', 'ByRunFileTypeSizeWithFlush']
    stepsListDictExpected = [mergeStepBHADRON, mergeStepCALIBRA, mergeStepPIDMDST, mergeStep]
    stepsInProdExpected = [[3], [4], [5], [6]]
    outputSEsExpected = ['Tier1-DST', 'Tier1-DST', 'Tier1-DST', 'Tier1-M-DST']
    prioritiesExpected = [1, 1, 1, 4]
    cpusExpected = [10, 10, 10, 100]
    groupSizesExpected = [1, 1, 1, 2]
    bkQueriesExpected = ['Full', 'Full', 'Full', 'fromPreviousProd']
    previousProdsExpected = [None, None, None, 1]
    removeInputsFlagExpected = [True, True, True, True]
    outputFileMasksExpected = ['', '', '', 'dst']
    outputFileStepsExpected = ['', '', '', '2']
    inputsExpected = [['aa'], ['aa'], ['aa'], ['bb']]
    inputDataPoliciesExpected = ['dl', 'dl', 'dl', 'pr']
    eventsExpected = [-1, -1, -1, -1]
    targetsExpected = ['Target1', 'Target1', 'Target1', 'Target2']
    multicoreExpected = ['False', 'False', 'False', 'True']
    processorsExpected = [2, 2, 2, 0]
    outputModeExpected = ['Local', 'Local', 'Local', 'Any']
    ancestorDepthsExpected = [1, 1, 1, 0]
    self.assertEqual(pr.prodsTypeList, prodsTypeListExpected)
    self.assertEqual(pr.plugins, pluginsExpected)
    self.assertEqual(pr.stepsListDict, stepsListDictExpected)
    self.assertEqual(pr.stepsInProds, stepsInProdExpected)
    self.assertEqual(pr.outputSEs, outputSEsExpected)
    self.assertEqual(pr.priorities, prioritiesExpected)
    self.assertEqual(pr.cpus, cpusExpected)
    self.assertEqual(pr.groupSizes, groupSizesExpected)
    self.assertEqual(pr.bkQueries, bkQueriesExpected)
    self.assertEqual(pr.previousProds, previousProdsExpected)
    self.assertEqual(pr.removeInputsFlags, removeInputsFlagExpected)
    self.assertEqual(pr.outputFileMasks, outputFileMasksExpected)
    self.assertEqual(pr.outputFileSteps, outputFileStepsExpected)
    self.assertEqual(pr.inputs, inputsExpected)
    self.assertEqual(pr.inputDataPolicies, inputDataPoliciesExpected)
    self.assertEqual(pr.events, eventsExpected)
    self.assertEqual(pr.targets, targetsExpected)
    self.assertEqual(pr.multicore, multicoreExpected)
    self.assertEqual(pr.processors, processorsExpected)
    self.assertEqual(pr.outputModes, outputModeExpected)
    self.assertEqual(pr.ancestorDepths, ancestorDepthsExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.prodsTypeList = ['MCSimulation', 'Merge']
    pr.plugins = ['', 'ByRunFileTypeSizeWithFlush']
    pr.stepsListDict = [stepMC, mergeStep]
    pr.stepsList = [123, 456]
    pr.stepsInProds = [[1], [2]]
    pr.outputSEs = ['Tier1-DST', 'Tier1-M-DST']
    pr.priorities = [1, 4]
    pr.cpus = [10, 100]
    pr.groupSizes = [1, 2]
    pr.removeInputsFlags = [False, True]
    pr.outputFileMasks = ['', 'dst']
    pr.outputFileSteps = ['', '2']
    pr.inputs = [[], []]
    pr.inputDataPolicies = ['', 'dl']
    pr.events = [100, -1]
    pr.bkQueries = ['', 'fromPreviousProd']
    pr.targets = ['Target1', 'Target2']
    pr.multicore = ['False', 'True']
    pr.processors = [2, 0]
    pr.outputModes = ['Local', 'Any']
    pr.ancestorDepths = [1, 0]
    pr._applyOptionalCorrections()
    prodsTypeListExpected = ['MCSimulation', 'Merge']
    pluginsExpected = ['', 'ByRunFileTypeSizeWithFlush']
    stepsListDictExpected = [stepMC, mergeStep]
    stepsInProdExpected = [[1], [2]]
    outputSEsExpected = ['Tier1-DST', 'Tier1-M-DST']
    prioritiesExpected = [1, 4]
    cpusExpected = [10, 100]
    groupSizesExpected = [1, 2]
    bkQueriesExpected = ['', 'fromPreviousProd']
    previousProdsExpected = [None, 1]
    removeInputsFlagExpected = [False, True]
    outputFileMasksExpected = ['', 'dst']
    outputFileStepsExpected = ['', '2']
    inputsExpected = [[], []]
    inputDataPoliciesExpected = ['', 'dl']
    eventsExpected = [100, -1]
    targetsExpected = ['Target1', 'Target2']
    multicoreExpected = ['False', 'True']
    processorsExpected = [2, 0]
    outputModeExpected = ['Local', 'Any']
    ancestorDepthsExpected = [1, 0]
    self.assertEqual(pr.prodsTypeList, prodsTypeListExpected)
    self.assertEqual(pr.plugins, pluginsExpected)
    self.assertEqual(pr.stepsListDict, stepsListDictExpected)
    self.assertEqual(pr.stepsInProds, stepsInProdExpected)
    self.assertEqual(pr.outputSEs, outputSEsExpected)
    self.assertEqual(pr.priorities, prioritiesExpected)
    self.assertEqual(pr.cpus, cpusExpected)
    self.assertEqual(pr.groupSizes, groupSizesExpected)
    self.assertEqual(pr.bkQueries, bkQueriesExpected)
    self.assertEqual(pr.previousProds, previousProdsExpected)
    self.assertEqual(pr.removeInputsFlags, removeInputsFlagExpected)
    self.assertEqual(pr.outputFileMasks, outputFileMasksExpected)
    self.assertEqual(pr.outputFileSteps, outputFileStepsExpected)
    self.assertEqual(pr.inputs, inputsExpected)
    self.assertEqual(pr.inputDataPolicies, inputDataPoliciesExpected)
    self.assertEqual(pr.events, eventsExpected)
    self.assertEqual(pr.targets, targetsExpected)
    self.assertEqual(pr.multicore, multicoreExpected)
    self.assertEqual(pr.processors, processorsExpected)
    self.assertEqual(pr.outputModes, outputModeExpected)
    self.assertEqual(pr.ancestorDepths, ancestorDepthsExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.prodsTypeList = ['MCSimulation', 'Merge', 'Merge']
    pr.plugins = ['', 'ByRunFileTypeSizeWithFlush', 'RootMerging']
    pr.stepsListDict = [stepMC, mergeStep, stepHistoMergingDict]
    pr.stepsList = [123, 456, 789]
    pr.stepsInProds = [[1], [2], [3]]
    pr.outputSEs = ['Tier1-DST', 'Tier1-M-DST', 'Tier1-M-DST']
    pr.priorities = [1, 4, 5]
    pr.cpus = [10, 100, 100]
    pr.groupSizes = [1, 2, 3]
    pr.removeInputsFlags = [False, True, True]
    pr.outputFileMasks = ['', 'dst', 'root']
    pr.outputFileSteps = ['', '2', '3']
    pr.inputs = [[], [], []]
    pr.inputDataPolicies = ['', 'dl', 'dl']
    pr.events = [100, -1, -1]
    pr.bkQueries = ['', 'fromPreviousProd', 'fromPreviousProd']
    pr.targets = ['Target1', 'Target2', 'T3']
    pr.multicore = ['False', 'True', 'True']
    pr.processors = [2, 0, 0]
    pr.outputModes = ['Local', 'Any', 'Any']
    pr.ancestorDepths = [1, 0, 0]
    pr._applyOptionalCorrections()
    prodsTypeListExpected = ['MCSimulation', 'Merge', 'Merge']
    pluginsExpected = ['', 'ByRunFileTypeSizeWithFlush', 'RootMerging']
    stepsListDictExpected = [stepMC, mergeStep, stepHistoMergingDict]
    stepsInProdExpected = [[1], [2], [3]]
    outputSEsExpected = ['Tier1-DST', 'Tier1-M-DST', 'Tier1-M-DST']
    prioritiesExpected = [1, 4, 5]
    cpusExpected = [10, 100, 100]
    groupSizesExpected = [1, 2, 3]
    bkQueriesExpected = ['', 'fromPreviousProd', 'fromPreviousProd']
    previousProdsExpected = [None, 1, 2]
    removeInputsFlagExpected = [False, True, True]
    outputFileMasksExpected = ['', 'dst', 'root']
    outputFileStepsExpected = ['', '2', '3']
    inputsExpected = [[], [], []]
    inputDataPoliciesExpected = ['', 'dl', 'dl']
    eventsExpected = [100, -1, -1]
    targetsExpected = ['Target1', 'Target2', 'T3']
    multicoreExpected = ['False', 'True', 'True']
    processorsExpected = [2, 0, 0]
    outputModeExpected = ['Local', 'Any', 'Any']
    ancestorDepthsExpected = [1, 0, 0]
    self.assertEqual(pr.prodsTypeList, prodsTypeListExpected)
    self.assertEqual(pr.plugins, pluginsExpected)
    self.assertEqual(pr.stepsListDict, stepsListDictExpected)
    self.assertEqual(pr.stepsInProds, stepsInProdExpected)
    self.assertEqual(pr.outputSEs, outputSEsExpected)
    self.assertEqual(pr.priorities, prioritiesExpected)
    self.assertEqual(pr.cpus, cpusExpected)
    self.assertEqual(pr.groupSizes, groupSizesExpected)
    self.assertEqual(pr.bkQueries, bkQueriesExpected)
    self.assertEqual(pr.previousProds, previousProdsExpected)
    self.assertEqual(pr.removeInputsFlags, removeInputsFlagExpected)
    self.assertEqual(pr.outputFileMasks, outputFileMasksExpected)
    self.assertEqual(pr.outputFileSteps, outputFileStepsExpected)
    self.assertEqual(pr.inputs, inputsExpected)
    self.assertEqual(pr.inputDataPolicies, inputDataPoliciesExpected)
    self.assertEqual(pr.events, eventsExpected)
    self.assertEqual(pr.targets, targetsExpected)
    self.assertEqual(pr.multicore, multicoreExpected)
    self.assertEqual(pr.processors, processorsExpected)
    self.assertEqual(pr.outputModes, outputModeExpected)
    self.assertEqual(pr.ancestorDepths, ancestorDepthsExpected)

  def test_getProdsDescriptionDict(self):
    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.stepsList = [123, 456, 456, 456, 789]
    pr.prodsTypeList = ['DataStripping', 'Merge', 'Merge', 'Merge']
    pr.removeInputsFlags = [False, True, True, True]
    pr.inputs = [[], [], [], []]
    pr.targets = ['', '', '', '']
    pr.groupSizes = [1, 1, 2, 3]
    pr.inputDataPolicies = ['download', 'download', 'download', 'download']
    pr.outputFileMasks = ['', '', '', '']
    pr.outputFileSteps = ['', '', '', '']
    pr.stepsInProds = [[1, 2], [3], [4], [5]]
    pr.bkQueries = ['Full', 'fromPreviousProd', 'fromPreviousProd', 'fromPreviousProd']
#     pr.outputSEs = ['Tier1-Buffer', 'Tier1-DST', 'Tier1-DST']
#     pr.specialOutputSEs = [{}, {}, {}]
    pr.outputSEsPerFileType = [{'T1': 'SE1'}, {'T1': 'SE1'}, {'T1': 'SE1'}, {'T1': 'SE1'}]
    pr.priorities = [5, 8, 9, 9]
    pr.cpus = [1000000, 300000, 10000, 1000]
    pr.plugins = ['ByRun', 'BySize', 'BySize', 'BySize']
    pr.previousProds = [None, 1, 1, 1]
    pr.events = [-1, -1, -1, -1]
    pr.multicore = ['False', 'False', 'True', 'True']
    pr.processors = [(4, 4), (2, 2), (0, 2), (0, 0)]
    pr.outputModes = ['Any', 'Local', 'Any', 'Any']
    pr.ancestorDepths = [0, 0, 0, 0]

    pr.stepsListDict = [{'StepId': 123, 'StepName': 'Stripping14-Stripping',
                         'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v2r2',
                         'OptionFiles': 'optsFiles', 'Visible': 'Yes', 'ExtraPackages': 'eps',
                         'ProcessingPass': 'procPass', 'OptionsFormat': '',
                         'DDDB': '', 'CONDDB': '123456', 'DQTag': '', 'isMulticore': 'False',
                         'fileTypesIn': ['SDST'],
                         'fileTypesOut':['BHADRON.DST', 'CALIBRATION.DST']},
                        {'StepId': 456, 'StepName': 'Merge',
                         'ApplicationName': 'LHCb', 'ApplicationVersion': 'v1r2',
                         'OptionFiles': 'optsFiles', 'Visible': 'Yes', 'ExtraPackages': 'eps',
                         'ProcessingPass': 'procPass', 'OptionsFormat': '',
                         'DDDB': '', 'CONDDB': '123456', 'DQTag': '', 'isMulticore': 'False',
                         'prodStepID': "456['BHADRON.DST']",
                         'fileTypesIn': ['BHADRON.DST'],
                         'fileTypesOut':['BHADRON.DST']},
                        {'StepId': 456, 'StepName': 'Merge',
                         'ApplicationName': 'LHCb', 'ApplicationVersion': 'v1r2',
                         'OptionFiles': 'optsFiles', 'Visible': 'Yes', 'ExtraPackages': 'eps',
                         'ProcessingPass': 'procPass', 'OptionsFormat': '',
                         'DDDB': '', 'CONDDB': '123456', 'DQTag': '', 'isMulticore': 'False',
                         'prodStepID': "456['CALIBRATION.DST']",
                         'fileTypesIn': ['CALIBRATION.DST'],
                         'fileTypesOut':['CALIBRATION.DST']},
                        {'StepId': 456, 'StepName': 'Merge',
                         'ApplicationName': 'LHCb', 'ApplicationVersion': 'v1r2',
                         'OptionFiles': 'optsFiles', 'Visible': 'Yes', 'ExtraPackages': 'eps',
                         'ProcessingPass': 'procPass', 'OptionsFormat': '',
                         'DDDB': '', 'CONDDB': '123456', 'DQTag': '', 'isMulticore': 'False',
                         'prodStepID': "456['PID.MDST']",
                         'fileTypesIn': ['PID.MDST'],
                         'fileTypesOut':['PID.MDST']},
                        {'StepId': 789, 'StepName': 'Merge',
                         'ApplicationName': 'Noether', 'ApplicationVersion': 'v1r2',
                         'OptionFiles': 'optsFiles', 'Visible': 'Yes', 'ExtraPackages': 'eps',
                         'ProcessingPass': 'procPass', 'OptionsFormat': '',
                         'DDDB': '', 'CONDDB': '123456', 'DQTag': '', 'isMulticore': 'False',
                         'prodStepID': "789['DAVINCIHIST', 'BRUNELHIST']",
                         'fileTypesIn': ['DAVINCIHIST', 'BRUNELHIST'],
                         'fileTypesOut':['ROOT']},
                        ]
    res = pr._getProdsDescriptionDict()

    self.maxDiff = None
    self.assertEqual(res, prodsDict)

  def test__addStepsToProd(self):

    stepsInProd = [{'StepId': 123, 'StepName': 'Stripping14-Stripping',
                    'ApplicationName': 'DaVinci', 'ApplicationVersion': 'v2r2', 'ExtraOptions': '',
                    'OptionFiles': 'optsFiles', 'Visible': 'Yes', 'ExtraPackages': 'eps',
                    'ProcessingPass': 'procPass', 'OptionsFormat': '', 'SystemConfig': '',
                    'DDDB': '', 'CONDDB': '123456', 'DQTag': '', 'isMulticore': 'N',
                    'prodStepID': "123['SDST']", 'mcTCK': '',
                    'visibilityFlag': [{'Visibility': 'N', 'FileType': 'BHADRON.DST'},
                                       {'Visibility': 'N', 'FileType': 'CALIBRATION.DST'}],
                    'fileTypesIn': ['SDST'],
                    'fileTypesOut':['BHADRON.DST', 'CALIBRATION.DST']},
                   {'StepId': 456, 'StepName': 'Merge',
                    'ApplicationName': 'LHCb', 'ApplicationVersion': 'v1r2', 'ExtraOptions': '',
                    'OptionFiles': 'optsFiles', 'Visible': 'Yes', 'ExtraPackages': 'eps',
                    'ProcessingPass': 'procPass', 'OptionsFormat': '',
                    'SystemConfig': 'x86_64+avx2-centos7-gcc7-opt',
                    'prodStepID': "456['BHADRON.DST', 'CALIBRATION.DST']",
                    'DDDB': '', 'CONDDB': '123456', 'DQTag': '', 'isMulticore': 'N', 'mcTCK': '',
                    'visibilityFlag': [{'Visibility': 'Y', 'FileType': 'BHADRON.DST'},
                                       {'Visibility': 'Y', 'FileType': 'CALIBRATION.DST'}],
                    'fileTypesIn': ['BHADRON.DST', 'CALIBRATION.DST'],
                    'fileTypesOut':['BHADRON.DST', 'CALIBRATION.DST']}
                   ]

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    prod = Production()
    prod = pr._addStepsToProd(prod, stepsInProd, True)
    self.assertEqual(prod.gaudiSteps, ['DaVinci_1', 'LHCb_2'])

  # def test__getStepsInProdDAG( self ):

  #   pr = ProductionRequest( self.bkClientFake, self.diracProdIn )
  #   # stepsListDict = {}
  #   # for prodDict in prodsDict.values():
  #   #   stepsInProdDAG = pr._getStepsInProdDAG(prodDict, stepsListDict)
  #   #   self.assertEqual(stepsInProdDAG.graph, {})

  #   stepsListDict = [step1Dict, mergeStepBHADRON]
  #   stepsInProdDAG = pr._getStepsInProdDAG(prodsDict[1], stepsListDict)

  #   self.assertEqual(
  #       stepsInProdDAG.graph,
  #       {frozenset([('DDDB', ''),
  #                   ('ExtraPackages', 'eps'),
  #                   ('OptionsFormat', ''),
  #                   ('visibilityFlag', frozenset([
  #                       {'Visible':'N', 'FileType': 'BHADRON.DST'},
  #                       {'Visible':'N', 'FileType': 'CALIBRATION.DST'},
  #                   ])),
  #                   ('fileTypesOut', frozenset(['CALIBRATION.DST', 'BHADRON.DST'])),
  #                   ('DQTag', ''),
  #                   ('CONDDB', '123456'),
  #                   ('mcTCK', ''),
  #                   ('ApplicationName', 'DaVinci'),
  #                   ('ApplicationVersion', 'v2r2'),
  #                   ('Visible', 'Yes'),
  #                   ('ProcessingPass', 'procPass'),
  #                   ('fileTypesIn', frozenset(['SDST'])),
  #                   ('isMulticore', 'N'),
  #                   ('SystemConfig', ''),
  #                   ('prodStepID', "123['SDST']"),
  #                   ('StepName', 'Stripping14-Stripping'),
  #                   ('StepId', 123),
  #                   ('OptionFiles', 'optsFiles'),
  #                   ('ExtraOptions', '')]): set([frozenset([
  #                       ('DDDB', 'head-20110302'),
  #                       ('ExtraPackages', 'AppConfig.v3r104'),
  #                       ('fileTypesOut', frozenset(['BHADRON.DST'])),
  #                       ('CONDDB', 'head-20110407'),
  #                       ('mcTCK', ''),
  #                       ('visibilityFlag', frozenset([''])),
  #                       ('ApplicationName', 'DaVinci'),
  #                       ('ApplicationVersion', 'v28r3p1'),
  #                       ('ProcessingPass', 'Merging'),
  #                       ('fileTypesIn', frozenset(['BHADRON.DST'])),
  #                       ('prodStepID', "456['BHADRON.DST']"),
  #                       ('StepName', 'Stripping14-Merging'),
  #                       ('StepId', 456),
  #                       ('SystemConfig', ''),
  #                       ('Usable', 'Yes'),
  #                       ('Visible', 'N'),
  #                       ('OptionFiles', '$APPCONFIGOPTS/Merging/DV-Stripping14-Merging.py')
  #                   ])]),
  #         frozenset([('DDDB', 'head-20110302'),
  #                   ('ExtraPackages', 'AppConfig.v3r104'),
  #                   ('fileTypesOut', frozenset(['BHADRON.DST'])),
  #                   ('CONDDB', 'head-20110407'),
  #                   ('mcTCK', ''),
  #                   ('visibilityFlag', frozenset([''])),
  #                   ('ApplicationName', 'DaVinci'),
  #                   ('ApplicationVersion', 'v28r3p1'),
  #                   ('ProcessingPass', 'Merging'),
  #                   ('fileTypesIn', frozenset(['BHADRON.DST'])),
  #                   ('prodStepID', "456['BHADRON.DST']"),
  #                   ('StepName', 'Stripping14-Merging'),
  #                   ('StepId', 456),
  #                   ('SystemConfig', ''),
  #                   ('Usable', 'Yes'),
  #                   ('Visible', 'N'),
  #                   ('OptionFiles', '$APPCONFIGOPTS/Merging/DV-Stripping14-Merging.py')]): set([]),
  #       }
  #     )

  def test__getBKKQuery(self):

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.dataTakingConditions = 'dataTC'
    pr.processingPass = 'procePass'
    pr.dqFlag = 'OK,AA, BB'
    pr.startRun = '123'
    pr.endRun = '456'
    res = pr._getBKKQuery()

    resExpected = {'DataTakingConditions': 'dataTC',
                   'ProcessingPass': 'procePass',
                   'FileType': '',
                   'EventType': '',
                   'ConfigName': 'test',
                   'ConfigVersion': 'certification',
                   'DataQualityFlag': 'OK;;;AA;;;BB',
                   'StartRun': 123,
                   'EndRun': 456
                   }
    self.assertEqual(res, resExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.dataTakingConditions = 'dataTC'
    pr.processingPass = 'procePass'
    pr.dqFlag = 'OK,AA, BB'
    pr.bkFileType = ['type1', 'type2']
    pr.startRun = '123'
    pr.endRun = '456'
    res = pr._getBKKQuery()
    resExpected = {'DataTakingConditions': 'dataTC',
                   'ProcessingPass': 'procePass',
                   'FileType': 'type1;;;type2',
                   'EventType': '',
                   'ConfigName': 'test',
                   'ConfigVersion': 'certification',
                   'DataQualityFlag': 'OK;;;AA;;;BB',
                   'StartRun': 123,
                   'EndRun': 456
                   }
    self.assertEqual(res, resExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.dataTakingConditions = 'dataTC'
    pr.processingPass = 'procePass'
    pr.dqFlag = 'OK,AA, BB'
    pr.bkFileType = ['type1']
    pr.startRun = '123'
    pr.endRun = '456'
    res = pr._getBKKQuery()
    resExpected = {'DataTakingConditions': 'dataTC',
                   'ProcessingPass': 'procePass',
                   'FileType': 'type1',
                   'EventType': '',
                   'ConfigName': 'test',
                   'ConfigVersion': 'certification',
                   'DataQualityFlag': 'OK;;;AA;;;BB',
                   'StartRun': 123,
                   'EndRun': 456
                   }
    self.assertEqual(res, resExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.dataTakingConditions = 'dataTC'
    pr.processingPass = 'procePass'
    pr.dqFlag = 'OK,AA, BB'
    pr.bkFileType = 'type1'
    pr.startRun = '123'
    pr.endRun = '456'
    res = pr._getBKKQuery()
    resExpected = {'DataTakingConditions': 'dataTC',
                   'ProcessingPass': 'procePass',
                   'FileType': 'type1',
                   'EventType': '',
                   'ConfigName': 'test',
                   'ConfigVersion': 'certification',
                   'DataQualityFlag': 'OK;;;AA;;;BB',
                   'StartRun': 123,
                   'EndRun': 456
                   }
    self.assertEqual(res, resExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.dataTakingConditions = 'dataTC'
    pr.processingPass = 'procePass'
    pr.dqFlag = 'OK,AA, BB'
    pr.bkFileType = 'type1, type2'
    pr.startRun = '123'
    pr.endRun = '456'
    res = pr._getBKKQuery()
    resExpected = {'DataTakingConditions': 'dataTC',
                   'ProcessingPass': 'procePass',
                   'FileType': 'type1;;;type2',
                   'EventType': '',
                   'ConfigName': 'test',
                   'ConfigVersion': 'certification',
                   'DataQualityFlag': 'OK;;;AA;;;BB',
                   'StartRun': 123,
                   'EndRun': 456
                   }
    self.assertEqual(res, resExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.dqFlag = 'OK,AA, BB'
    res = pr._getBKKQuery('fromPreviousProd', ['type'], 123)
    resExpected = {'ProductionID': 123,
                   'FileType': 'type',
                   'DataQualityFlag': 'OK;;;AA;;;BB'}
    self.assertEqual(res, resExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.dqFlag = 'OK,AA, BB'
    res = pr._getBKKQuery('fromPreviousProd', ['type'], 123)
    resExpected = {'ProductionID': 123,
                   'FileType': 'type',
                   'DataQualityFlag': 'OK;;;AA;;;BB'}
    self.assertEqual(res, resExpected)

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.dqFlag = 'OK,AA, BB'
    res = pr._getBKKQuery('fromPreviousProd', ['type1', 'type2'], 123)
    resExpected = {'ProductionID': 123,
                   'FileType': 'type1;;;type2',
                   'DataQualityFlag': 'OK;;;AA;;;BB'}
    self.assertEqual(res, resExpected)

  def test__splitIntoProductionSteps(self):

    r_p = _splitIntoProductionSteps(stepStripp)
    r_exp = [{'ApplicationName': 'DaVinci', 'Usable': 'Yes', 'StepId': 123, 'ApplicationVersion': 'v28r3p1',
              'ExtraPackages': 'AppConfig.v3r104', 'StepName': 'Stripping14-Merging', 'SystemConfig': '',
              'ProcessingPass': 'Merging', 'Visible': 'N', 'DDDB': 'head-20110302', 'mcTCK': '',
              'OptionFiles': '$APPCONFIGOPTS/Merging/DV-Stripping14-Merging.py', 'CONDDB': 'head-20110407',
              'visibilityFlag': [],
              'fileTypesIn': ['SDST'],
              'fileTypesOut': ['BHADRON.DST', 'CALIBRATION.DST', 'CHARM.MDST', 'CHARMCOMPLETEEVENT.DST']}]

    self.assertEqual(r_p, r_exp)

    r_p = _splitIntoProductionSteps(mergeStep)
    r_exp = [{'ApplicationName': 'DaVinci', 'Usable': 'Yes', 'StepId': 456, 'ApplicationVersion': 'v28r3p1',
              'ExtraPackages': 'AppConfig.v3r104', 'StepName': 'Stripping14-Merging', 'SystemConfig': '',
              'prodStepID': "456['BHADRON.DST']", 'mcTCK': '',
              'ProcessingPass': 'Merging', 'Visible': 'N', 'DDDB': 'head-20110302',
              'OptionFiles': '$APPCONFIGOPTS/Merging/DV-Stripping14-Merging.py', 'CONDDB': 'head-20110407',
              'visibilityFlag': [],
              'fileTypesIn': ['BHADRON.DST'],
              'fileTypesOut': ['BHADRON.DST']},
             {'ApplicationName': 'DaVinci', 'Usable': 'Yes', 'StepId': 456, 'ApplicationVersion': 'v28r3p1',
              'ExtraPackages': 'AppConfig.v3r104', 'StepName': 'Stripping14-Merging', 'SystemConfig': '',
              'prodStepID': "456['CALIBRATION.DST']", 'mcTCK': '',
              'ProcessingPass': 'Merging', 'Visible': 'N', 'DDDB': 'head-20110302',
              'OptionFiles': '$APPCONFIGOPTS/Merging/DV-Stripping14-Merging.py', 'CONDDB': 'head-20110407',
              'visibilityFlag': [],
              'fileTypesIn': ['CALIBRATION.DST'],
              'fileTypesOut': ['CALIBRATION.DST']},
             {'ApplicationName': 'DaVinci', 'Usable': 'Yes', 'StepId': 456, 'ApplicationVersion': 'v28r3p1',
              'ExtraPackages': 'AppConfig.v3r104', 'StepName': 'Stripping14-Merging', 'SystemConfig': '',
              'prodStepID': "456['PID.MDST']", 'mcTCK': '',
              'ProcessingPass': 'Merging', 'Visible': 'N', 'DDDB': 'head-20110302',
              'OptionFiles': '$APPCONFIGOPTS/Merging/DV-Stripping14-Merging.py', 'CONDDB': 'head-20110407',
              'visibilityFlag': [],
              'fileTypesIn': ['PID.MDST'],
              'fileTypesOut': ['PID.MDST']}
             ]

    self.assertEqual(r_p, r_exp)


class ProductionRequestFailure(ClientTestCase):

  def test_resolveStepsFailure(self):
    pr = ProductionRequest(self.bkClientMock, self.diracProdIn)
    pr.stepsList = ['123']
    self.bkClientMock.getAvailableSteps.return_value = {'OK': False, 'Message': 'error'}
    self.assertRaises(ValueError, pr.resolveSteps)

    self.bkClientMock.getStepInputFiles.return_value = {'OK': False, 'Message': 'error'}
    self.assertRaises(ValueError, pr.resolveSteps)


class ProductionRequestFullChain(ClientTestCase):

  def test_full(self):

    pr = ProductionRequest(self.bkClientFake, self.diracProdIn)
    pr.logger.setLevel('VERBOSE')

    stepsList = ['123']
    stepsList.append('456')
    stepsList.append('')
    stepsList.append('')
    stepsList.append('')
    pr.stepsList = stepsList
    pr.compressionLvl = ['LOW'] * 4
    pr.outputVisFlag = [{'1': 'N'}, {'2': 'N'}]
    pr.resolveSteps()

    pr.appendName = '1'
    pr.configName = 'MC'
    pr.configVersion = 'MC11a'

    pr.events = ['100', '-1']

    pr.eventsToProduce = 10000

    pr.eventType = '11124001'
    pr.parentRequestID = '34'
    pr.requestID = '35'

    pr.prodGroup = 'Sim05/Trig0x40760037Flagged/Reco12a/Stripping17Flagged'
    pr.dataTakingConditions = 'Beam3500GeV-2011-MagDown-Nu2-EmNoCuts'

    pr.prodsTypeList = ['Stripping', 'Merge']
    pr.outputSEs = ['Tier1_MC-DST', 'Tier1_MC-DST']
    pr.specialOutputSEs = [{}, {}]
    pr.stepsInProds = [range(1, len(pr.stepsList)), [len(pr.stepsList)]]
    pr.removeInputsFlags = [False, True]
    pr.priorities = [1, 6]
    pr.cpus = [1000, 100]
    pr.outputModes = ['Local', 'Any']
    pr.outputFileMasks = ['FOO', '']
    pr.outputFileSteps = ['2', '']
    pr.targets = ['Tier2', '']
    pr.groupSizes = [1, 5]
    pr.plugins = ['', 'BySize']
    pr.inputDataPolicies = ['', 'protocol']
    pr.inputs = [[], []]

    res = pr.buildAndLaunchRequest()

    self.assertEqual(res, {'OK': True, 'Value': [321, 321, 321]})

# ###########################################################################
# Test Suite run
# ###########################################################################


if __name__ == '__main__':
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(ClientTestCase)
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ProductionRequestSuccess))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ProductionRequestFailure))
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(ProductionRequestFullChain))
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)

# EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Test_Resources_Storage_LHCbOnlineStorage."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import mock
import unittest

import LHCbDIRAC.Resources.Storage.LHCbOnlineStorage as moduleTested

__RCSID__ = "$Id$"

################################################################################


class LHCbOnlineStorage_TestCase(unittest.TestCase):
  def setUp(self):
    """Setup."""

    # Mock external libraries / modules not interesting for the unit test
    mock_xmlrpclib = mock.Mock()
    mock_xmlrpclib.Server.return_value('')
    self.mock_xmlrpclib = mock_xmlrpclib

    # Add mocks to moduleTested
    moduleTested.xmlrpclib = self.mock_xmlrpclib

    self.moduleTested = moduleTested
    self.testClass = self.moduleTested.LHCbOnlineStorage

  def tearDown(self):
    """TearDown."""
    del self.testClass
    del self.moduleTested
    del self.mock_xmlrpclib


################################################################################
# Tests


class LHCbOnlineStorage_Success(LHCbOnlineStorage_TestCase):
  def test_instantiate(self):
    """tests that we can instantiate one object of the tested class."""

    resource = self.testClass('storageName', {
        'Protocol': 'protocol',
        'Path': 'path',
        'Host': 'host',
        'Port': 'port',
        'SpaceToken': 'spaceToken',
        'WSUrl': 'wspath'
    })

    self.assertEqual('LHCbOnlineStorage', resource.__class__.__name__)

  def test_init(self):
    """tests that the init method does what it should do."""

    resource = self.testClass('storageName', {
        'Protocol': 'protocol',
        'Path': 'path',
        'Host': 'host',
        'Port': 'port',
        'SpaceToken': 'spaceToken',
        'WSUrl': 'wspath'
    })

    self.assertEqual('storageName', resource.name)
    self.assertEqual('protocol', resource.protocolParameters['Protocol'])
    self.assertEqual('path', resource.protocolParameters['Path'])
    self.assertEqual('host', resource.protocolParameters['Host'])
    self.assertEqual('port', resource.protocolParameters['Port'])
    self.assertEqual('spaceToken', resource.protocolParameters['SpaceToken'])
    self.assertEqual('wspath', resource.protocolParameters['WSUrl'])

  def test_getParameters(self):
    """tests the output of getParameters method."""

    resource = self.testClass('storageName', {
        'Protocol': 'protocol',
        'Path': 'path',
        'Host': 'host',
        'Port': 'port',
        'SpaceToken': 'spaceToken',
        'WSUrl': 'wspath'
    })

    res = resource.getParameters()

    self.assertEqual('storageName', res['StorageName'])
    self.assertEqual('protocol', res['Protocol'])
    self.assertEqual('path', res['Path'])
    self.assertEqual('host', res['Host'])
    self.assertEqual('port', res['Port'])
    self.assertEqual('spaceToken', res['SpaceToken'])
    self.assertEqual('wspath', res['WSUrl'])

  def test_getFileSize(self):
    """tests the output of getFileSize."""

    resource = self.testClass('storageName', {
        'Protocol': 'protocol',
        'Path': 'path',
        'Host': 'host',
        'Port': 'port',
        'SpaceToken': 'spaceToken',
        'WSUrl': 'wspath'
    })

    res = resource.getFileSize({})
    self.assertEqual(False, res['OK'])

    res = resource.getFileSize([])
    self.assertEqual(False, res['OK'])

    res = resource.getFileSize(['A', 'B'])
    self.assertEqual(True, res['OK'])
    self.assertEqual({'A': 0, 'B': 0}, res['Value']['Successful'])
    self.assertEqual({}, res['Value']['Failed'])

    res = resource.getFileSize({'A': 1, 'B': {}})
    self.assertEqual(True, res['OK'])
    self.assertEqual({'A': 0, 'B': 0}, res['Value']['Successful'])
    self.assertEqual({}, res['Value']['Failed'])

  def test_removeFile(self):
    """tests the output of removeFile."""

    resource = self.testClass('storageName', {
        'Protocol': 'protocol',
        'Path': 'path',
        'Host': 'host',
        'Port': 'port',
        'SpaceToken': 'spaceToken',
        'WSUrl': 'wspath'
    })

    # no need for mock here, we don't reach it

    res = resource.removeFile({})
    self.assertEqual(False, res['OK'])

    res = resource.removeFile([])
    self.assertEqual(False, res['OK'])

    resource.server.endMigratingFileBulk.return_value = (1, {})

    res = resource.removeFile(['A', 'B'])
    self.assertEqual(True, res['OK'])
    self.assertEqual({'A': True, 'B': True}, res['Value']['Successful'])
    self.assertEqual({}, res['Value']['Failed'])

    resource.server.endMigratingFileBulk.return_value = (1, {'B': 'Did not work'})

    res = resource.removeFile(['A', 'B'])
    self.assertEqual(True, res['OK'])
    self.assertEqual({'A': True}, res['Value']['Successful'])
    self.assertEqual(['B'], list(res['Value']['Failed']))

    resource.server.endMigratingFileBulk.side_effect = Exception('Boom!')
    res = resource.removeFile(['A', 'B'])
    # FIXME: This should return S_ERROR !!
    self.assertEqual(True, res['OK'])
    self.assertEqual({}, res['Value']['Successful'])
    self.assertEqual(['A', 'B'], list(res['Value']['Failed']))

  def test_retransferOnlineFile(self):
    """tests output of retransferOnlineFile."""

    resource = self.testClass('storageName', {
        'Protocol': 'protocol',
        'Path': 'path',
        'Host': 'host',
        'Port': 'port',
        'SpaceToken': 'spaceToken',
        'WSUrl': 'wspath'
    })
    resource.server.errorMigratingFile.return_value = (1, 0)

    res = resource.retransferOnlineFile({})
    self.assertEqual(False, res['OK'])

    res = resource.retransferOnlineFile([])
    self.assertEqual(False, res['OK'])

    res = resource.retransferOnlineFile(['A', 'B'])
    self.assertEqual(True, res['OK'])
    self.assertEqual({'A': True, 'B': True}, res['Value']['Successful'])
    self.assertEqual({}, res['Value']['Failed'])

    # side_effect does not work very well, cooked a workaround
    _myValues = [(0, 1), (1, 0)]

    def _side_effect(_pfn):
      return _myValues.pop()

    resource.server.errorMigratingFile.side_effect = _side_effect

    res = resource.retransferOnlineFile(['A', 'B'])
    self.assertEqual(True, res['OK'])
    self.assertEqual({'A': True}, res['Value']['Successful'])
    self.assertEqual(['B'], list(res['Value']['Failed']))

    resource.server.errorMigratingFile.side_effect = Exception('Boom!')
    res = resource.retransferOnlineFile(['A', 'B'])
    # FIXME: This should return S_ERROR !!
    self.assertEqual(True, res['OK'])
    self.assertEqual({}, res['Value']['Successful'])
    self.assertEqual(['A', 'B'], list(res['Value']['Failed']))



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""remove output of production."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import sys

from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  from DIRAC.Core.Base.Script import parseCommandLine
  parseCommandLine()

  from DIRAC import exit as DIRACExit
  from DIRAC import gLogger
  from LHCbDIRAC.TransformationSystem.Agent.TransformationCleaningAgent import TransformationCleaningAgent
  from LHCbDIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

  if len(sys.argv) < 2:
    print('Usage: dirac-production-remove-output transID [transID] [transID]')
    DIRACExit(1)
  else:
    try:
      transIDs = [int(arg) for arg in sys.argv[1:]]
    except BaseException:
      print('Invalid list of productions')
      DIRACExit(1)

  agent = TransformationCleaningAgent('Transformation/TransformationCleaningAgent',
                                      'Transformation/TransformationCleaningAgent',
                                      'dirac-production-remove-output')
  agent.initialize()

  client = TransformationClient()
  for transID in transIDs:
    res = client.getTransformationParameters(transID, ['Status'])
    if not res['OK']:
      gLogger.error("Failed to determine transformation status")
      gLogger.error(res['Message'])
      continue
    status = res['Value']
    if status not in ['RemovingFiles', 'RemovingOutput', 'ValidatingInput', 'Active']:
      gLogger.error("The transformation is in %s status and the outputs cannot be removed" % status)
      continue
    agent.removeTransformationOutput(transID)


if __name__ == "__main__":
  main()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Simple merging module for MDF files."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import shlex

from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.Utilities.Subprocess import systemCall
from DIRAC.Resources.Catalog.PoolXMLCatalog import PoolXMLCatalog

from LHCbDIRAC.Workflow.Modules.ModuleBase import ModuleBase

__RCSID__ = "$Id$"


class MergeMDF(ModuleBase):
  """To be used in normal workflows."""

  #############################################################################
  def __init__(self, bkClient=None, dm=None):
    """Module initialization."""
    self.log = gLogger.getSubLogger("MergeMDF")
    super(MergeMDF, self).__init__(self.log, bkClientIn=bkClient, dm=dm)

    self.version = __RCSID__

    self.outputLFN = ''
    # List all input parameters here
    self.stepInputData = []
    self.poolXMLCatName = 'pool_xml_catalog.xml'
    self.applicationName = 'cat'

  #############################################################################
  def _resolveInputVariables(self):
    """By convention the module parameters are resolved here."""

    super(MergeMDF, self)._resolveInputVariables()
    super(MergeMDF, self)._resolveInputStep()

  #############################################################################

  def execute(self, production_id=None, prod_job_id=None, wms_job_id=None,
              workflowStatus=None, stepStatus=None,
              wf_commons=None, step_commons=None,
              step_number=None, step_id=None):
    """Main execution function."""

    try:

      super(MergeMDF, self).execute(self.version,
                                    production_id, prod_job_id, wms_job_id,
                                    workflowStatus, stepStatus,
                                    wf_commons, step_commons,
                                    step_number, step_id)

      poolCat = PoolXMLCatalog(self.poolXMLCatName)

      self._resolveInputVariables()

      stepOutputs, stepOutputTypes, _histogram = self._determineOutputs()

      logLines = ['#' * len(self.version), self.version, '#' * len(self.version)]

      localInputs = [str(list(poolCat.getPfnsByLfn(x)['Replicas'].values())[0]) for x in self.stepInputData]
      inputs = ' '.join(localInputs)
      cmd = 'cat %s > %s' % (inputs, self.outputFilePrefix + '.' + stepOutputTypes[0])
      logLines.append('\nExecuting merge operation...')
      self.log.info('Executing "%s"' % cmd)
      result = systemCall(timeout=600, cmdSeq=shlex.split(cmd))
      if not result['OK']:
        self.log.error(result)
        logLines.append('Merge operation failed with result:\n%s' % result)
        return S_ERROR('Problem Executing Application')

      status = result['Value'][0]
      stdout = result['Value'][1]
      stderr = result['Value'][2]
      self.log.info(stdout)
      if stderr:
        self.log.error(stderr)

      if status:
        msg = 'Non-zero status %s while executing "%s"' % (status, cmd)
        self.log.info(msg)
        logLines.append(msg)
        return S_ERROR('Problem Executing Application')

      self.log.info("Going to manage %s output" % self.applicationName)
      self._manageAppOutput(stepOutputs)

      # Still have to set the application status e.g. user job case.
      self.setApplicationStatus('%s %s Successful' % (self.applicationName, self.applicationVersion))

      # Write to log file
      msg = 'Produced merged MDF file'
      self.log.info(msg)
      logLines.append(msg)
      logLines = [str(i) for i in logLines]
      logLines.append('#EOF')
      with open(self.applicationLog, 'w') as fopen:
        fopen.write('\n'.join(logLines) + '\n')

      return S_OK('%s %s Successful' % (self.applicationName, self.applicationVersion))

    except Exception as e:  # pylint:disable=broad-except
      self.log.exception("Failure in MergeMDF execute module", lException=e)
      return S_ERROR(str(e))

    finally:
      super(MergeMDF, self).finalize(self.version)

# EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#EOF#



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Utilities to parse the XML Generator Logs."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
import io
import ast
import json
import six
import xmltodict


def counterJson(listCounters):
  '''returns a dictionary containing counters
    :param list listCounters: list containing all the counter nodes
  '''
  dictCounters = dict()
  if isinstance(listCounters, dict):
    listCounters = [listCounters]
  for counter in listCounters:
    dictCounters[counter['@name']] = int(counter['value'])
  return dictCounters


def efficiencyJson(listEfficiencies):
  '''returns a dictionary containing efficiencies
    :param list listEfficiencies: list containing all the efficiency nodes
  '''
  dictEfficiencies = dict()
  if isinstance(listEfficiencies, dict):
    listEfficiencies = [listEfficiencies]
  for efficiency in listEfficiencies:
    dictEfficiencies[efficiency['@name']] = {
        'after': int(efficiency['after']),
        'before': int(efficiency['before']),
        'error': float(efficiency['error']),
        'value': float(efficiency['value'])
    }
  return dictEfficiencies


def fractionJson(listFractions):
  '''returns a dictionary containing fractions
    :param list listFractions: list containing all the fraction nodes
  '''
  dictFractions = dict()
  if isinstance(listFractions, dict):
    listFractions = [listFractions]
  for fraction in listFractions:
    dictFractions[fraction['@name']] = {
        'number': int(fraction['number']),
        'error': float(fraction['error']),
        'value': float(fraction['value'])
    }
  return dictFractions


def crossSectionJson(listCrossSections):
  '''returns a dictionary containing cross sections
    :param list listCrossSections: list containing all the cross section nodes
  '''
  dictCrossSections = dict()
  if isinstance(listCrossSections, dict):
    listCrossSections = [listCrossSections]
  for crossSection in listCrossSections:
    dictCrossSections[crossSection['description'][1:-1]] = {
        'ID': int(crossSection['@id']),
        'generated': int(crossSection['generated']),
        'value': float(crossSection['value'])
    }
  return dictCrossSections


def methodGeneratorJson(listMethods, listGenerators, numberEventTypes):
  '''returns a dictionary containing the generator for each method
    :param list listMethods: list containing all the methods
    :param list listGenerators: list containing all the generators
    :param int numberEventTypes: number of event types in the generator log
  '''
  dictMethods = dict()
  if numberEventTypes > 1:
    for i, method in enumerate(listMethods):
      dictMethods[method] = listGenerators[i]
  else:
    dictMethods[listMethods] = listGenerators
  return dictMethods


class GeneratorLog(object):
  def __init__(self):
    pass

  def generatorLogJson(self, fileName):
    '''converts the xml Generator Log into json format
    '''
    dictElements = dict()
    dictGenerator = dict()

    with io.open('GeneratorLog.xml', 'r') as fp:
      fileLines = fp.readlines()

    fileLines = fileLines[fileLines.index('<generatorCounters>\n'):fileLines.index('</generatorCounters>\n') + 1]
    xmlText = ''.join(fileLines)
    numberEventTypes = xmlText.count('<eventType>')
    if numberEventTypes > 1:
      # Taking the first set of nodes
      xmlText = xmlText.split('<eventType>')[0] + '<eventType>' + xmlText.split('<eventType>')[1] + '<method>' + xmlText.split('<eventType>')[-1].split('<method>', 1)[-1]  # noqa
    xmlText = xmlText.replace('-nan', '-1')
    dicto = xmltodict.parse(xmlText)
    jsonData = ast.literal_eval(json.dumps(dicto))

    listCounters = jsonData['generatorCounters'].get('counter', [])
    listEfficiencies = jsonData['generatorCounters'].get('efficiency', [])
    listFractions = jsonData['generatorCounters'].get('fraction', [])
    listCrossSections = jsonData['generatorCounters'].get('crosssection', [])
    listMethods = jsonData['generatorCounters']['method']
    listGenerators = jsonData['generatorCounters']['generator']

    dictElements['counter'] = counterJson(listCounters)
    dictElements['efficiency'] = efficiencyJson(listEfficiencies)
    dictElements['fraction'] = fractionJson(listFractions)
    dictElements['crossSection'] = crossSectionJson(listCrossSections)
    dictElements['method'] = methodGeneratorJson(listMethods, listGenerators, numberEventTypes)

    dictGenerator['generatorCounters'] = dictElements

    with io.open(fileName, 'w', encoding="utf-8") as fp:
        fp.write(six.text_type(json.dumps(dictGenerator, indent=2)))

    return dictGenerator



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Test_AS_Client_Type_DataStorage."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import unittest

__RCSID__ = "$Id$"

dummyResults = {}


class DummyReturn(object):

  def __init__(self, *args, **kwargs):
    pass

  def __getattr__(self, name):
    return self.dummyMethod

  def dummyMethod(self, *args, **kwargs):
    return dummyResults[self.__class__.__name__]


class dBaseAccountingType(DummyReturn):
  pass

################################################################################


class DataStorage_TestCase(unittest.TestCase):

  def setUp(self):
    """Setup."""

    # We need the proper software, and then we overwrite it.
    import LHCbDIRAC.AccountingSystem.Client.Types.DataStorage as moduleTested
    moduleTested.BaseAccountingType = dBaseAccountingType
    moduleTested.DataStorage.__bases__ = (dBaseAccountingType, )

    self.accountingType = moduleTested.DataStorage

  def tearDown(self):
    """TearDown."""
    del self.accountingType


class DataStorage_Success(DataStorage_TestCase):

  def test_instantiate(self):
    """tests that we can instantiate one object of the tested class."""
    global dummyResults
    dummyResults['DataStorage'] = None

    accountingType = self.accountingType()
    self.assertEqual('DataStorage', accountingType.__class__.__name__)

################################################################################


if __name__ == '__main__':
  suite = unittest.defaultTestLoader.loadTestsFromTestCase(DataStorage_TestCase)
  suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(DataStorage_Success))
  testResult = unittest.TextTestRunner(verbosity=2).run(suite)



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""
:mod: RAWIntegrityHandler

.. module: RAWIntegrityHandler

:synopsis: DISET interface to the RAWIntegrityDB.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
# imports
import six
# from DIRAC
from DIRAC import gLogger, S_OK, S_ERROR
from DIRAC.Core.DISET.RequestHandler import RequestHandler
# from LHCbDIRAC
from LHCbDIRAC.DataManagementSystem.DB.RAWIntegrityDB import RAWIntegrityDB

__RCSID__ = "$Id$"

# global instance of the RAWIntegrityDB
gRAWIntegrityDB = False


def initializeRAWIntegrityHandler(_serviceInfo):
  """handler initialisation."""

  global gRAWIntegrityDB
  gRAWIntegrityDB = RAWIntegrityDB()

  # Create DB tables if needed
  return gRAWIntegrityDB._checkTable()


class RAWIntegrityHandler(RequestHandler):
  """
  .. class:: RAWIntegrityHandler
  """

  types_addFile = [
      six.string_types, six.string_types, six.integer_types,
      six.string_types, six.string_types, six.string_types,
  ]

  @staticmethod
  def export_addFile(lfn, pfn, size, se, guid, checksum):
    """Add a file to the RAW integrity DB."""
    try:
      gLogger.info("RAWIntegrityHandler.addFile: Attempting to add %s to the gRAWIntegrityDB." % lfn)
      res = gRAWIntegrityDB.addFile(lfn, pfn, size, se, guid, checksum)
      return res
    except Exception as x:
      errStr = "RAWIntegrityHandler.addFile: Exception while adding file to gRAWIntegrityDB."
      gLogger.exception(errStr, lException=x)
      return S_ERROR(errStr)

  types_getGlobalStatistics = []

  @staticmethod
  def export_getGlobalStatistics():
    """Get global file statistics."""
    try:
      gLogger.info("Attempting to get global statistics.")
      res = gRAWIntegrityDB.getGlobalStatistics()
      if not res['OK']:
        gLogger.error("getGlobalStatistics: Failed to get global statistics", res['Message'])
      else:
        gLogger.info("getGlobalStatistics: Obtained global statistics")
      return res
    except Exception as x:
      errStr = "getGlobalStatistics: Exception while getting global statistics."
      gLogger.exception(errStr, lException=x)
      return S_ERROR(errStr)

  types_getFileSelections = []

  @staticmethod
  def export_getFileSelections():
    """Get the possible selections available."""
    try:
      gLogger.info("Attempting to get selections.")
      res = gRAWIntegrityDB.getFileSelections()
      if not res['OK']:
        gLogger.error("getFileSelections: Failed to get file selections", res['Message'])
      else:
        gLogger.info("getFileSelections: Obtained file selections")
      return res
    except Exception as x:
      errStr = "getFileSelections: Exception while getting file selections."
      gLogger.exception(errStr, lException=x)
      return S_ERROR(errStr)

  types_getFilesSummaryWeb = [dict, list, six.integer_types, six.integer_types]

  @staticmethod
  def export_getFilesSummaryWeb(selectDict, sortList, startItem, maxItems):
    """Get the file information according to conventions."""
    resultDict = {}
    startDate = selectDict.get('FromDate', None)
    if startDate:
      del selectDict['FromDate']
    endDate = selectDict.get('ToDate', None)
    if endDate:
      del selectDict['ToDate']
    # Sorting instructions. Only one for the moment.
    if sortList:
      orderAttribute = sortList[0][0] + ":" + sortList[0][1]
    else:
      orderAttribute = None
    res = gRAWIntegrityDB.selectFiles(selectDict, orderAttribute=orderAttribute,
                                      newer=startDate, older=endDate)
    if not res['OK']:
      return S_ERROR('Failed to select jobs: ' + res['Message'])
    # Get the files and the counters correctly
    fileList = res['Value']
    nFiles = len(fileList)
    resultDict['TotalRecords'] = nFiles
    if nFiles == 0:
      return S_OK(resultDict)
    iniFile = startItem
    lastFile = iniFile + maxItems
    if iniFile >= nFiles:
      return S_ERROR('Item number out of range')
    if lastFile > nFiles:
      lastFile = nFiles
    summaryFileList = fileList[iniFile:lastFile]
    # Prepare the standard format
    resultDict['ParameterNames'] = ['lfn', 'pfn', 'size', 'storageelement', 'guid',
                                    'checksum', 'startTime', 'endTime', 'status']
    records = []
    statusCountDict = {}
    for lfn, pfn, size, se, guid, checksum, submit, complete, status in fileList:
      if status not in statusCountDict:
        statusCountDict[status] = 0
      statusCountDict[status] += 1
    for fileTuple in summaryFileList:
      lfn, pfn, size, se, guid, checksum, submit, complete, status = fileTuple
      startTime = str(submit)
      endTime = str(complete)
      records.append((lfn, pfn, size, se, guid, checksum, startTime, endTime, status))
    resultDict['Records'] = records
    resultDict['Extras'] = statusCountDict
    return S_OK(resultDict)

  types_getStatistics = [six.string_types, dict]

  @staticmethod
  def export_getStatistics(attribute, selectDict):
    """Get job statistics distribution per attribute value with a given
    selection."""
    startDate = selectDict.get('FromDate', None)
    if startDate:
      del selectDict['FromDate']
    # For backward compatibility
    if startDate is None:
      startDate = selectDict.get('LastUpdate', None)
      if startDate:
        del selectDict['LastUpdate']
    endDate = selectDict.get('ToDate', None)
    if endDate:
      del selectDict['ToDate']

    result = gRAWIntegrityDB.getCounters('Files', [attribute], selectDict,
                                         newer=startDate,
                                         older=endDate)

    resultDict = {}
    if result['OK']:
      for cDict, count in result['Value']:
        resultDict[cDict[attribute]] = count

    return S_OK(resultDict)



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Returns run ranges, split by conditions and by run gaps or time interval between them."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  from LHCbDIRAC.DataManagementSystem.Client.DMScript import Script

  Script.registerSwitch('', 'Activity=', 'Specify the BK activity (e.g. Collision15)')
  Script.registerSwitch('', 'Runs=', 'Run range or list (can be used with --Activity to reduce the run range)')
  Script.registerSwitch('', 'Fast', 'Include runs even if no FULL stream is present (much faster)')
  Script.registerSwitch('', 'DQFlag=', 'Specify the DQ flag (default: all)')
  Script.registerSwitch('', 'RunGap=', 'Gap between run ranges, in number of runs')
  Script.registerSwitch('', 'TimeGap=', 'Gap between run ranges, in number of days')
  Script.setUsageMessage(__doc__ + '\n'.join([
      'Usage:',
      '  %s [option|cfgfile] ... ' % Script.scriptName]))

  Script.parseCommandLine(ignoreErrors=True)

  from LHCbDIRAC.BookkeepingSystem.Client.ScriptExecutors import executeRunInfo
  executeRunInfo('Ranges')


if __name__ == "__main__":
  main()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Module to remove input data files for given workflow.

Initially written for use after merged outputs have been successfully
uploaded to an SE.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.RequestManagementSystem.Client.Operation import Operation
from DIRAC.RequestManagementSystem.Client.File import File
from LHCbDIRAC.Workflow.Modules.ModuleBase import ModuleBase


class RemoveInputData(ModuleBase):

  #############################################################################

  def __init__(self, bkClient=None, dm=None):
    """Module initialization."""

    self.log = gLogger.getSubLogger("RemoveInputData")
    super(RemoveInputData, self).__init__(self.log, bkClientIn=bkClient, dm=dm)

    self.version = __RCSID__

    # List all parameters here
    self.inputDataList = []

  #############################################################################

  def _resolveInputVariables(self):
    """By convention the module parameters are resolved here."""

    super(RemoveInputData, self)._resolveInputVariables()

  #############################################################################

  def execute(self, production_id=None, prod_job_id=None, wms_job_id=None,
              workflowStatus=None, stepStatus=None,
              wf_commons=None, step_commons=None,
              step_number=None, step_id=None):
    """Main execution function."""

    try:

      super(RemoveInputData, self).execute(self.version, production_id, prod_job_id, wms_job_id,
                                           workflowStatus, stepStatus,
                                           wf_commons, step_commons, step_number, step_id)

      if not self._checkWFAndStepStatus():
        return S_OK()

      if not self._enableModule():
        return S_OK()

      self._resolveInputVariables()

      # Try to remove the file list with failover if necessary
      failover = []
      self.log.info('Attempting dm.removeFile("%s")' % (self.inputDataList))
      result = self.dataManager.removeFile(self.inputDataList)
      self.log.verbose(result)
      if not result['OK']:
        self.log.error('Could not remove files with message:\n"%s"\n\
        Will set removal requests just in case.' % (result['Message']))
        failover = self.inputDataList
      try:
        if result['Value']['Failed']:
          failureDict = result['Value']['Failed']
          if failureDict:
            self.log.info('Not all files were successfully removed, see "LFN : reason" below\n%s' % (failureDict))
          failover = list(failureDict)
      except KeyError:
        self.log.error('Setting files for removal request to be the input data: %s' % self.inputDataList)
        failover = self.inputDataList

      for lfn in failover:
        self.__setFileRemovalRequest(lfn)

      return S_OK('Input Data Removed')

    except Exception as e:  # pylint:disable=broad-except
      self.log.exception("Failure in RemoveInputData execute module", lException=e)
      self.setApplicationStatus(e)
      return S_ERROR(str(e))

    finally:
      super(RemoveInputData, self).finalize(self.version)

  #############################################################################

  def __setFileRemovalRequest(self, lfn):
    """Sets a removal request for a file including all replicas."""
    self.log.info('Setting file removal request for %s' % lfn)
    removeFile = Operation()
    removeFile.Type = 'RemoveFile'
    rmFile = File()
    rmFile.LFN = lfn
    removeFile.addFile(rmFile)
    self.request.addOperation(removeFile)



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""Get the metadata of a (list of) LFNs from the FC."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
  from LHCbDIRAC.DataManagementSystem.Client.DMScript import DMScript, Script

  dmScript = DMScript()
  dmScript.registerFileSwitches()
  dmScript.registerSiteSwitches()
  Script.setUsageMessage(__doc__ + '\n'.join([
      'Usage:',
      '  %s [option|cfgfile] ... [LFN[,LFN2[,LFN3...]]]' % Script.scriptName,
      'Arguments:',
      '  LFN:      Logical File Name or file containing LFNs']))
  Script.parseCommandLine(ignoreErrors=True)

  from LHCbDIRAC.DataManagementSystem.Client.ScriptExecutors import executeLfnMetadata
  from DIRAC import exit
  exit(executeLfnMetadata(dmScript))


if __name__ == "__main__":
  main()



###############################################################################
# (c) Copyright 2019 CERN for the benefit of the LHCb Collaboration           #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""
   LHCbDIRAC - LHCb extension of DIRAC

   References:
    DIRAC: https://github.com/DIRACGrid/DIRAC
    LHCbDIRAC: https://gitlab.cern.ch/lhcb-dirac/LHCbDIRAC

   The distributed data production and analysis system of LHCb.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os

from pkgutil import extend_path
__path__ = extend_path(__path__, __name__)  # pylint: disable=redefined-builtin

import six

rootPath = os.path.dirname(os.path.realpath(__path__[0]))

# Define Version
if six.PY3:
  from pkg_resources import get_distribution, DistributionNotFound

  try:
    __version__ = get_distribution(__name__).version
    version = __version__
  except DistributionNotFound:
    # package is not installed
    version = "Unknown"
else:
  majorVersion = 10
  minorVersion = 2
  patchLevel = 6
  preVersion = 0

  version = "v%sr%s" % (majorVersion, minorVersion)
  __version__ = "%s.%s" % (majorVersion, minorVersion)
  buildVersion = "v%dr%d" % (majorVersion, minorVersion)
  if patchLevel:
    version = "%sp%s" % (version, patchLevel)
    __version__ += ".%s" % patchLevel
    buildVersion = "%s build %s" % (buildVersion, patchLevel)
  if preVersion:
    version = "%s-pre%s" % (version, preVersion)
    __version__ += "a%s" % preVersion
    buildVersion = "%s pre %s" % (buildVersion, preVersion)


def extension_metadata():
  return {
      "primary_extension": True,
      "priority": 100,
      "setups": {
          "LHCb-Production": "dips://lhcb-conf-dirac.cern.ch:9135/Configuration/Server",
          "LHCb-Certification": "dips://lhcb-cert-dirac.cern.ch:9135/Configuration/Server",
      },
      "default_setup": "LHCb-Production",
  }
